"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const inversify_1 = require("inversify");
const Cli_1 = require("../components/cli/Cli");
const Exception_1 = require("../components/error/Exception");
const tmp = require("tmp");
const InterpolateApplicationConfiguration_1 = require("./InterpolateApplicationConfiguration");
const InterpolateApplicationUI_1 = require("./InterpolateApplicationUI");
const InterpolateApplicationContextReader_1 = require("./InterpolateApplicationContextReader");
const YamlService_1 = require("../components/yaml/YamlService");
const fs = require("fs");
const Path = require("path");
const traverse = require("traverse");
const YamlLazyValue_1 = require("../components/yaml/YamlLazyValue");
let InterpolateApplication = class InterpolateApplication {
    constructor($cli, $configuration, $ui, $contextReader, $yaml) {
        this.$cli = $cli;
        this.$configuration = $configuration;
        this.$ui = $ui;
        this.$contextReader = $contextReader;
        this.$yaml = $yaml;
    }
    async run() {
        this.validateEnvironment();
        this.loadApplicationContext();
        this.interpolate();
    }
    validateEnvironment() {
        const commandExists = require('command-exists').sync;
        this.$configuration.requiredCommands.map((command) => {
            if (!commandExists(command)) {
                throw new Exception_1.default(`Missing required command!\nPlease see README for the details!`, { Command: command });
            }
        });
    }
    validateContext(context) {
        if (!context.inputFile) {
            throw "Invalid input file!";
        }
    }
    loadApplicationContext() {
        const context = this.$contextReader.readContext();
        if (context.quiet)
            this.$cli.quiet = true;
        switch (context.logLevel) {
            case "error":
                this.$cli.levels.error = true;
                this.$cli.levels.warn = false;
                this.$cli.levels.info = false;
                this.$cli.levels.debug = false;
                this.$cli.levels.trace = false;
                break;
            case "warn":
                this.$cli.levels.error = true;
                this.$cli.levels.warn = true;
                this.$cli.levels.info = false;
                this.$cli.levels.debug = false;
                this.$cli.levels.trace = false;
                break;
            case "info":
                this.$cli.levels.error = true;
                this.$cli.levels.warn = true;
                this.$cli.levels.info = true;
                this.$cli.levels.debug = false;
                this.$cli.levels.trace = false;
                break;
            case "debug":
                this.$cli.levels.error = true;
                this.$cli.levels.warn = true;
                this.$cli.levels.info = true;
                this.$cli.levels.debug = true;
                this.$cli.levels.trace = false;
                break;
            case "trace":
                this.$cli.levels.error = true;
                this.$cli.levels.warn = true;
                this.$cli.levels.info = true;
                this.$cli.levels.debug = true;
                this.$cli.levels.trace = true;
                break;
        }
        if (context.help) {
            this.$ui.showApplicationHelp();
            process.exit(0);
        }
        else {
            this.$ui.showApplicationInfo(context);
            try {
                this.validateContext(context);
            }
            catch (error) {
                this.$cli.logError(error);
                this.$ui.showApplicationHelp();
                process.exit(1);
            }
        }
        this.$configuration.context.tempDir = context.tempDir;
        this.$configuration.context.logLevel = context.logLevel;
        this.$configuration.context.inputFile = context.inputFile;
        this.$configuration.context.output = context.output;
        this.$configuration.context.help = context.help;
        this.$configuration.context.quiet = context.quiet;
        this.$configuration.context.mergeFiles = context.mergeFiles;
        this.$configuration.context.contextFile = context.contextFile;
        if (!this.$configuration.context.tempDir) {
            tmp.setGracefulCleanup();
            this.$configuration.context.tempDir = tmp.dirSync({ unsafeCleanup: true }).name;
        }
    }
    interpolate() {
        const context = this.$configuration.context;
        const pathInput = context.inputFile;
        const mergeFiles = context.mergeFiles || [];
        const contextFile = context.contextFile;
        const cwd = Path.dirname(pathInput);
        let inputContent = this.$yaml.parseFile(pathInput, {
            context: contextFile ? this.$yaml.parseFile(contextFile) : undefined
        });
        traverse(inputContent).forEach(function (element) {
            if (element instanceof YamlLazyValue_1.default) {
                this.update("!lazy " + element.value);
            }
        });
        let source = this.$yaml.stringify(inputContent, { path: pathInput })
            .replace(/"<<-": '!lazy /g, '<<: \'')
            .replace(/<<-: '!lazy /g, '<<: \'')
            .replace(/"<<-": "!lazy /g, '<<: \"')
            .replace(/<<-: "!lazy /g, '<<: \"')
            .replace(/: '!lazy /g, ': \'')
            .replace(/: "!lazy /g, ': \"')
            .replace(/- '!lazy /g, '- \'')
            .replace(/- "!lazy /g, '- \"')
            .replace(/"<<-": !lazy /g, '<<: ')
            .replace(/<<-: !lazy /g, '<<: ')
            .replace(/: !lazy /g, ': ')
            .replace(/- !lazy /g, '- ')
            .replace(/"<<-": /g, "<<: ")
            .replace(/<<-: /g, "<<: ");
        const pathInterpolated = cwd + "/~~" + Path.basename(pathInput) + ".interpolate.yaml";
        fs.writeFileSync(pathInterpolated, source);
        source = this.$yaml.spiffFiles([pathInterpolated, ...mergeFiles], {
            cwd: cwd
        });
        fs.unlinkSync(pathInterpolated);
        if (context.output) {
            fs.writeFileSync(context.output, source);
        }
        else {
            console.log(source);
        }
    }
};
InterpolateApplication = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(Cli_1.default)),
    __param(1, inversify_1.inject(InterpolateApplicationConfiguration_1.default)),
    __param(2, inversify_1.inject(InterpolateApplicationUI_1.default)),
    __param(3, inversify_1.inject(InterpolateApplicationContextReader_1.default)),
    __param(4, inversify_1.inject(YamlService_1.default)),
    __metadata("design:paramtypes", [Cli_1.default,
        InterpolateApplicationConfiguration_1.default,
        InterpolateApplicationUI_1.default,
        InterpolateApplicationContextReader_1.default,
        YamlService_1.default])
], InterpolateApplication);
exports.default = InterpolateApplication;
//# sourceMappingURL=InterpolateApplication.js.map