"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const inversify_1 = require("inversify");
const OpsApplicationContextReader_1 = require("./OpsApplicationContextReader");
const Cli_1 = require("../components/cli/Cli");
const OpsApplicationUI_1 = require("./OpsApplicationUI");
const fs = require("fs");
const _ = require("lodash");
const OpsApplicationConfiguration_1 = require("./OpsApplicationConfiguration");
const TaskExecutionService_1 = require("../components/tasks/TaskExecutionService");
const TaskExecutionError_1 = require("../components/tasks/TaskExecutionError");
const EchoTaskExecutor_1 = require("../components/tasks/executors/echo/EchoTaskExecutor");
const TerraformSourcesFormatterTaskExecutor_1 = require("../components/tasks/executors/terraform/TerraformSourcesFormatterTaskExecutor");
const TerragruntConfigurationGeneratorTaskExecutor_1 = require("../components/tasks/executors/terraform/TerragruntConfigurationGeneratorTaskExecutor");
const TerraformTaskExecutor_1 = require("../components/tasks/executors/terraform/TerraformTaskExecutor");
const ManifestTaskExecutor_1 = require("../components/tasks/executors/manifest/ManifestTaskExecutor");
const TerraformSourcesImporterTaskExecutor_1 = require("../components/tasks/executors/terraform/TerraformSourcesImporterTaskExecutor");
const OpsTaskExecutor_1 = require("../components/tasks/executors/ops/OpsTaskExecutor");
const TerraformVariablesGeneratorTaskExecutor_1 = require("../components/tasks/executors/terraform/TerraformVariablesGeneratorTaskExecutor");
const FSTaskExecutor_1 = require("../components/tasks/executors/fs/FSTaskExecutor");
const Exception_1 = require("../components/error/Exception");
const Models = require("../domain/Models");
const ModulesService_1 = require("../components/modules/ModulesService");
const TerraformSectionSourcesGeneratorTaskExecutor_1 = require("../components/tasks/executors/terraform/TerraformSectionSourcesGeneratorTaskExecutor");
const TerraformTemplateSourcesGeneratorTaskExecutor_1 = require("../components/tasks/executors/terraform/TerraformTemplateSourcesGeneratorTaskExecutor");
const TerraformRemoteStatesGeneratorTaskExecutor_1 = require("../components/tasks/executors/terraform/TerraformRemoteStatesGeneratorTaskExecutor");
const ExecTaskExecutor_1 = require("../components/tasks/executors/exec/ExecTaskExecutor");
const tmp = require("tmp");
const Moment = require("moment");
let OpsApplication = class OpsApplication {
    constructor($cli, $configuration, $ui, $contextReader, $modulesService, $taskExecutionService, $opsTaskExecutor, $fsTaskExecutor, $manifestTaskExecutor, $echoTaskExecutor, $execTaskExecutor, $terraformVariablesGeneratorTaskExecutor, $terraformSectionSourcesGeneratorTaskExecutor, $terraformTemplateSourcesGeneratorTaskExecutor, $terraformSourcesImporterTaskExecutor, $terraformSourcesFormatterTaskExecutor, $terragruntConfigurationGeneratorTaskExecutor, $terraformTaskExecutor, $terraformRemoteStatesGeneratorTaskExecutor) {
        this.$cli = $cli;
        this.$configuration = $configuration;
        this.$ui = $ui;
        this.$contextReader = $contextReader;
        this.$modulesService = $modulesService;
        this.$taskExecutionService = $taskExecutionService;
        this.$opsTaskExecutor = $opsTaskExecutor;
        this.$fsTaskExecutor = $fsTaskExecutor;
        this.$manifestTaskExecutor = $manifestTaskExecutor;
        this.$echoTaskExecutor = $echoTaskExecutor;
        this.$execTaskExecutor = $execTaskExecutor;
        this.$terraformVariablesGeneratorTaskExecutor = $terraformVariablesGeneratorTaskExecutor;
        this.$terraformSectionSourcesGeneratorTaskExecutor = $terraformSectionSourcesGeneratorTaskExecutor;
        this.$terraformTemplateSourcesGeneratorTaskExecutor = $terraformTemplateSourcesGeneratorTaskExecutor;
        this.$terraformSourcesImporterTaskExecutor = $terraformSourcesImporterTaskExecutor;
        this.$terraformSourcesFormatterTaskExecutor = $terraformSourcesFormatterTaskExecutor;
        this.$terragruntConfigurationGeneratorTaskExecutor = $terragruntConfigurationGeneratorTaskExecutor;
        this.$terraformTaskExecutor = $terraformTaskExecutor;
        this.$terraformRemoteStatesGeneratorTaskExecutor = $terraformRemoteStatesGeneratorTaskExecutor;
    }
    async run() {
        try {
            const time = +new Date();
            this.validateEnvironment();
            this.registerTaskExecutors();
            this.loadApplicationContext();
            await this.executeTarget();
            this.$cli.logHeader("Execution completed in", Moment.duration((+new Date) - time, "milliseconds").humanize(), "(" + ((+new Date) - time) + "ms)");
        }
        catch (error) {
            this.$cli.logError(error);
            this.$cli.logError("To re-run application please execute:\n", _.flatten([
                "ops",
                this.$configuration.context.help ?
                    ["--help"] : [],
                this.$configuration.context.targetModule ?
                    ["-m", this.$configuration.context.targetModule] : [],
                this.$configuration.context.bindingModule ?
                    ["-b", this.$configuration.context.bindingModule] : [],
                this.$configuration.context.targetLifecycle ?
                    ["-l", this.$configuration.context.targetLifecycle] : [],
                this.$configuration.context.targetTask ?
                    ["-t", this.$configuration.context.targetTask] : [],
                Object.keys(this.$configuration.context.inputs).length ?
                    ["-i", JSON.stringify(JSON.stringify(this.$configuration.context.inputs))] : [],
                this.$configuration.context.logLevel ?
                    ["-v", this.$configuration.context.logLevel] : [],
            ]).join(" "));
            process.exit(1);
        }
    }
    validateEnvironment() {
        const commandExists = require('command-exists').sync;
        this.$configuration.requiredCommands.map((command) => {
            if (!commandExists(command)) {
                throw new Exception_1.default(`Missing required command!\nPlease see README for the details!`, { Command: command });
            }
        });
    }
    validateContext(context) {
        if (!context.targetModule.length)
            throw "Module name is not specified!";
        if (!context.targetTask && !context.targetLifecycle)
            throw "Lifecycle name or task name is not specified!";
        if (!context.modulesRoot.length)
            throw "Path to modules is not specified!";
        if (!fs.existsSync(context.modulesRoot))
            throw "Path to modules doesn't exists!";
        if (!fs.lstatSync(context.modulesRoot).isDirectory())
            throw "Path to modules is not directory!";
    }
    loadApplicationContext() {
        const context = this.$contextReader.readContext();
        if (context.quiet) {
            this.$cli.quiet = true;
        }
        switch (context.logLevel) {
            case "error":
                this.$cli.levels.error = true;
                this.$cli.levels.warn = false;
                this.$cli.levels.info = false;
                this.$cli.levels.debug = false;
                this.$cli.levels.trace = false;
                break;
            case "warn":
                this.$cli.levels.error = true;
                this.$cli.levels.warn = true;
                this.$cli.levels.info = false;
                this.$cli.levels.debug = false;
                this.$cli.levels.trace = false;
                break;
            case "info":
                this.$cli.levels.error = true;
                this.$cli.levels.warn = true;
                this.$cli.levels.info = true;
                this.$cli.levels.debug = false;
                this.$cli.levels.trace = false;
                break;
            case "debug":
                this.$cli.levels.error = true;
                this.$cli.levels.warn = true;
                this.$cli.levels.info = true;
                this.$cli.levels.debug = true;
                this.$cli.levels.trace = false;
                break;
            case "trace":
                this.$cli.levels.error = true;
                this.$cli.levels.warn = true;
                this.$cli.levels.info = true;
                this.$cli.levels.debug = true;
                this.$cli.levels.trace = true;
                break;
        }
        if (context.help) {
            this.$ui.showApplicationHelp();
            process.exit(0);
        }
        else {
            this.$ui.showApplicationInfo(context);
            try {
                this.validateContext(context);
            }
            catch (error) {
                this.$cli.logError(error);
                this.$ui.showApplicationHelp();
                process.exit(1);
            }
        }
        this.$configuration.context.tempDir = context.tempDir;
        this.$configuration.context.logLevel = context.logLevel;
        this.$configuration.context.bindingModule = context.bindingModule;
        this.$configuration.context.modulesRoot = context.modulesRoot;
        this.$configuration.context.targetTask = context.targetTask;
        this.$configuration.context.targetModule = context.targetModule;
        this.$configuration.context.targetLifecycle = context.targetLifecycle;
        this.$configuration.context.help = context.help;
        this.$configuration.context.quiet = context.quiet;
        this.$configuration.context.inputs = context.inputs;
        this.$configuration.context.continueFrom = context.continueFrom;
        if (!this.$configuration.context.tempDir) {
            tmp.setGracefulCleanup();
            this.$configuration.context.tempDir = tmp.dirSync({ unsafeCleanup: true }).name;
        }
        this.$modulesService.setModulesRoot(context.modulesRoot);
    }
    async executeTarget() {
        const context = this.$configuration.context;
        const bindingModuleName = context.bindingModule;
        let bindingModule;
        if (bindingModuleName) {
            this.$cli.logTrace("Initializing registry for binding:", bindingModuleName);
            this.$modulesService.createRegistry("binding");
            if (!this.$modulesService.hasModule(bindingModuleName, "binding"))
                throw `Binding '${bindingModuleName}' is not found!`;
            bindingModule = await this.$modulesService.getModule(bindingModuleName, "binding");
            this.$modulesService.createRegistry("application", {
                binding: bindingModule.serializeAsBinding()
            });
        }
        else {
            this.$cli.logTrace("Initializing registry for application");
            this.$modulesService.createRegistry("application");
        }
        const targetModuleName = context.targetModule;
        const targetLifecycleName = context.targetLifecycle;
        const targetTask = context.targetTask;
        if (!this.$modulesService.hasModule(targetModuleName, "application"))
            throw `Module '${targetModuleName}' is not found!`;
        const targetModule = await this.$modulesService.getModule(targetModuleName, "application");
        if (targetLifecycleName && !targetModule.lifecycles.has(targetLifecycleName))
            throw `Module '${targetModuleName}' doesn't have lifecycle '${targetLifecycleName}'!`;
        if (targetTask && !targetModule.tasks.has(targetTask))
            throw `Module '${targetModuleName}' doesn't have task '${targetTask}'!`;
        if (!targetModule.metadata.enabled)
            throw `Module '${targetModuleName}' is not enabled!`;
        try {
            //this.$cli.logLifecycleStart(`Performing ${targetModuleName} ${targetLifecycleName ? `lifecycle:${targetLifecycleName}` : ""} ${targetTask ? `task:${targetTask}` : ""}`);
            const task = new Models.ModuleTask("ops", new Models.ModuleTaskMetadata(true, "ops", true, "Internal ops task"), {
                selector: {
                    id: targetModuleName
                },
                execution: {
                    type: targetLifecycleName ? "lifecycle" : "task",
                    targets: targetLifecycleName ? [targetLifecycleName] : [targetTask],
                    ignoreErrors: context.ignoreErrors,
                    continueFrom: context.continueFrom
                },
                context: {
                    binding: bindingModule ? bindingModule.serializeAsBinding() : undefined
                }
            });
            await this.$taskExecutionService.executeTask(task, context, targetModule, bindingModule);
            this.$cli.logLifecycleEnd();
        }
        catch (error) {
            if (error instanceof TaskExecutionError_1.default) {
                this.$cli.logNamedYamlObject(error, `Module: ${targetModuleName}`);
            }
            throw error;
        }
    }
    registerTaskExecutors() {
        [
            this.$opsTaskExecutor,
            this.$fsTaskExecutor,
            this.$manifestTaskExecutor,
            this.$echoTaskExecutor,
            this.$execTaskExecutor,
            this.$terraformVariablesGeneratorTaskExecutor,
            this.$terraformSectionSourcesGeneratorTaskExecutor,
            this.$terraformTemplateSourcesGeneratorTaskExecutor,
            this.$terraformSourcesImporterTaskExecutor,
            this.$terraformSourcesFormatterTaskExecutor,
            this.$terragruntConfigurationGeneratorTaskExecutor,
            this.$terraformTaskExecutor,
            this.$terraformRemoteStatesGeneratorTaskExecutor
        ].forEach((executor) => this.$taskExecutionService.registerExecutor(executor));
    }
};
OpsApplication = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(Cli_1.default)),
    __param(1, inversify_1.inject(OpsApplicationConfiguration_1.default)),
    __param(2, inversify_1.inject(OpsApplicationUI_1.default)),
    __param(3, inversify_1.inject(OpsApplicationContextReader_1.default)),
    __param(4, inversify_1.inject(ModulesService_1.default)),
    __param(5, inversify_1.inject(TaskExecutionService_1.default)),
    __param(6, inversify_1.inject(OpsTaskExecutor_1.default)),
    __param(7, inversify_1.inject(FSTaskExecutor_1.default)),
    __param(8, inversify_1.inject(ManifestTaskExecutor_1.default)),
    __param(9, inversify_1.inject(EchoTaskExecutor_1.default)),
    __param(10, inversify_1.inject(ExecTaskExecutor_1.default)),
    __param(11, inversify_1.inject(TerraformVariablesGeneratorTaskExecutor_1.default)),
    __param(12, inversify_1.inject(TerraformSectionSourcesGeneratorTaskExecutor_1.default)),
    __param(13, inversify_1.inject(TerraformTemplateSourcesGeneratorTaskExecutor_1.default)),
    __param(14, inversify_1.inject(TerraformSourcesImporterTaskExecutor_1.default)),
    __param(15, inversify_1.inject(TerraformSourcesFormatterTaskExecutor_1.default)),
    __param(16, inversify_1.inject(TerragruntConfigurationGeneratorTaskExecutor_1.default)),
    __param(17, inversify_1.inject(TerraformTaskExecutor_1.default)),
    __param(18, inversify_1.inject(TerraformRemoteStatesGeneratorTaskExecutor_1.default)),
    __metadata("design:paramtypes", [Cli_1.default,
        OpsApplicationConfiguration_1.default,
        OpsApplicationUI_1.default,
        OpsApplicationContextReader_1.default,
        ModulesService_1.default,
        TaskExecutionService_1.default,
        OpsTaskExecutor_1.default,
        FSTaskExecutor_1.default,
        ManifestTaskExecutor_1.default,
        EchoTaskExecutor_1.default,
        ExecTaskExecutor_1.default,
        TerraformVariablesGeneratorTaskExecutor_1.default,
        TerraformSectionSourcesGeneratorTaskExecutor_1.default,
        TerraformTemplateSourcesGeneratorTaskExecutor_1.default,
        TerraformSourcesImporterTaskExecutor_1.default,
        TerraformSourcesFormatterTaskExecutor_1.default,
        TerragruntConfigurationGeneratorTaskExecutor_1.default,
        TerraformTaskExecutor_1.default,
        TerraformRemoteStatesGeneratorTaskExecutor_1.default])
], OpsApplication);
exports.default = OpsApplication;
//# sourceMappingURL=OpsApplication.js.map