"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const parseCliArguments = require("command-line-args");
const fs = require("fs");
const Path = require("path");
const OpsApplicationCliArguments_1 = require("./OpsApplicationCliArguments");
const inversify_1 = require("inversify");
let OpsApplicationContextReader = class OpsApplicationContextReader {
    readContext() {
        const args = parseCliArguments(OpsApplicationCliArguments_1.default, { camelCase: true });
        const modulesRoot = this.sanitizePathManifests(args.pathManifests
            || process.env.OPS_PATH_MODULES
            || "");
        const bindingModule = this.sanitizeModule(args.binding
            || process.env.OPS_BINDING_MODULE
            || "");
        const targetModule = this.sanitizeModule(args.module
            || process.env.OPS_TARGET_MODULE
            || (function () {
                const cwd = Path.normalize(process.cwd());
                if (cwd.startsWith(modulesRoot)) {
                    if (fs.existsSync(`${cwd}/module.yml`) || fs.existsSync(`${cwd}/module.yaml`)) {
                        return "." + cwd.substr(modulesRoot.length);
                    }
                }
            }())
            || "");
        const targetLifecycle = this.sanitizeLifecycle(args.lifecycle
            || process.env.OPS_TARGET_LIFECYCLE
            || "");
        const targetTask = this.sanitizeTask(args.task
            || process.env.OPS_TARGET_TASK
            || "");
        const logLevel = args.logLevel
            || process.env.OPS_LOG_LEVEL
            || "info";
        const inputs = ((args.inputs || process.env.OPS_INPUTS || "").length ?
            JSON.parse(args.inputs || process.env.OPS_INPUTS) :
            {});
        const cmdTerraform = process.env.OPS_CMD_TERRAFORM || "terraform";
        const cmdTerragrunt = process.env.OPS_CMD_TERRAGRUNT || "terragrunt";
        const cmdSpiff = process.env.OPS_CMD_SPIFF || "spiff++";
        const cmdAWS = process.env.OPS_CMD_AWS || "aws";
        const continueFrom = args.continueFrom || "";
        return {
            tempDir: "",
            bindingModule,
            targetModule,
            targetLifecycle,
            targetTask,
            modulesRoot,
            logLevel,
            inputs,
            continueFrom,
            ignoreErrors: args.ignoreErrors || false,
            quiet: args.quiet || false,
            help: args.help || false,
            cmdTerraform: cmdTerraform,
            cmdTerragrunt: cmdTerragrunt,
            cmdSpiff: cmdSpiff,
            cmdAWS: cmdAWS
        };
    }
    sanitizeModule(name) {
        name = name.trim();
        ["/", "/module.yaml", "/module.yml"].forEach((postfix) => {
            if (name.toLowerCase().endsWith(postfix.toLowerCase())) {
                name = name.substr(0, name.length - postfix.length);
            }
        });
        if (!name.length)
            return name;
        if (!name.startsWith(".")) {
            name = "./" + name;
        }
        name = name.replace(/\/\//g, "/");
        return name;
    }
    sanitizeLifecycle(name) {
        return name.trim();
    }
    sanitizeTask(name) {
        return name.trim();
    }
    sanitizePathManifests(path) {
        return Path.normalize(path.trim());
    }
};
OpsApplicationContextReader = __decorate([
    inversify_1.injectable()
], OpsApplicationContextReader);
exports.default = OpsApplicationContextReader;
//# sourceMappingURL=OpsApplicationContextReader.js.map