"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const util = require("util");
const cli_highlight_1 = require("cli-highlight");
const yaml = require("yaml");
const CliLogLevels_1 = require("./CliLogLevels");
const inversify_1 = require("inversify");
const { Console } = require('console');
const fullConsole = new Console({
    stdout: process.stderr,
    stderr: process.stderr,
    colorMode: true
});
const quietConsole = new Console({
    stdout: process.stdout,
    stderr: process.stderr,
    colorMode: false
});
let Cli = class Cli {
    constructor(levels = new CliLogLevels_1.default(), quiet = false) {
        this.levels = levels;
        this.quiet = quiet;
    }
    get console() {
        return this.quiet ? quietConsole : fullConsole;
    }
    ;
    log(...args) {
        this.console.log(...args);
    }
    logProcessOutput(...args) {
        this.console.log(...args.map((arg) => arg
            .trim()
            .split("\n")
            .map((line) => line.trim())
            .join("\n")
            .trim()));
    }
    dump(object) {
        this.console.log(util.inspect(object, false, null, true /* enable colors */));
    }
    logInfo(...args) {
        if (this.quiet)
            return;
        this.console.log(chalk_1.default.green(["  -", ...this.serializeArgs(args)].join(" ")));
    }
    logHeader(...args) {
        if (this.quiet)
            return;
        this.console.log(chalk_1.default.green(["===", ...this.serializeArgs(args)].join(" ")));
    }
    logTaskStart(...args) {
        if (this.quiet)
            return;
        this.console.log(chalk_1.default.white(["  $", ...this.serializeArgs(args)].join(" ")));
    }
    logLifecycleStart(...args) {
        if (this.quiet)
            return;
        this.console.log(chalk_1.default.magentaBright(["——>", ...this.serializeArgs(args)].join(" ")));
    }
    logModuleProcessingStart(...args) {
        if (this.quiet)
            return;
        this.console.log(chalk_1.default.magentaBright([">>>", ...this.serializeArgs(args)].join(" ")));
    }
    logTaskEnd(...args) {
        if (this.quiet)
            return;
        //this.console.log(cli.bold(["<", ...this.serializeArgs(args)].join(" ")));
    }
    logLifecycleEnd(...args) {
        if (this.quiet)
            return;
        //this.console.log(cli.bold(["<——", ...this.serializeArgs(args)].join(" ")));
    }
    logDebug(...args) {
        if (this.quiet || !this.levels.debug)
            return;
        const prefix = "   ";
        this.console.log(chalk_1.default.dim([
            prefix,
            ...this.serializeArgs(args)
        ].join(" ").replace(/\n/g, "\n" + prefix)));
    }
    logTrace(...args) {
        if (this.quiet || !this.levels.trace)
            return;
        const prefix = "   ";
        this.console.log(chalk_1.default.dim([
            prefix,
            ...this.serializeArgs(args)
        ].join(" ").replace(/\n/g, "\n" + prefix)));
    }
    logError(...args) {
        this.console.log("\n" + chalk_1.default.red(["ERR", ...this.serializeArgs(args)].join(" ")));
    }
    logWarn(...args) {
        if (this.quiet)
            return;
        this.console.log(chalk_1.default.yellow(["<!>", ...this.serializeArgs(args)].join(" ")));
    }
    logYamlObject(content, force = false) {
        this.logYaml(yaml.stringify(content, {
            customTags: [
                {
                    class: Function,
                    tag: '!js',
                    stringify: (item) => {
                        return JSON.stringify(item.value.toString());
                    }
                },
                {
                    class: Function,
                    tag: '!include.js',
                    stringify: (item) => {
                        return JSON.stringify(item.value.toString());
                    }
                }
            ]
        }), force);
    }
    logNamedYamlObject(content, name = "", force = false) {
        this.logNamedYaml(yaml.stringify(content, {
            customTags: [
                {
                    class: Function,
                    tag: '!js',
                    stringify: (item) => {
                        return JSON.stringify(item.value.toString());
                    }
                },
                {
                    class: Function,
                    tag: '!include.js',
                    stringify: (item) => {
                        return JSON.stringify(item.value.toString());
                    }
                }
            ]
        }), name, force);
    }
    logYaml(yaml, force = false) {
        if (!force && (this.quiet || !this.levels.trace)) {
            return;
        }
        const content = cli_highlight_1.default(yaml, { language: 'yaml', ignoreIllegals: true });
        this.console.log(`---
${content.trim()}
`);
    }
    logNamedYaml(yaml, name = "", force = false) {
        if (!force && (this.quiet || !this.levels.trace)) {
            return;
        }
        const content = cli_highlight_1.default(yaml, { language: 'yaml', ignoreIllegals: true });
        this.console.log(`---
# ${name}
${content.trim()}
`);
    }
    logHcl(hcl, name = "", force = false) {
        if (!force && (this.quiet || !this.levels.trace)) {
            return;
        }
        const content = cli_highlight_1.default(hcl, { language: 'json', ignoreIllegals: true });
        this.console.log(`${name}
---
${content.trim()}
`);
    }
    serializeArgs(args) {
        return args.map((arg) => {
            if (typeof arg == "string" || typeof arg == "number") {
                return arg;
            }
            else if (arg instanceof Array) {
                return "\n" + yaml.stringify(arg, {
                    customTags: [
                        {
                            class: Function,
                            tag: '!js',
                            stringify: (item) => {
                                return JSON.stringify(item.value.toString());
                            }
                        },
                        {
                            class: Function,
                            tag: '!include.js',
                            stringify: (item) => {
                                return JSON.stringify(item.value.toString());
                            }
                        }
                    ]
                });
            }
            else {
                return util.inspect(arg, false, null, true);
            }
        });
    }
};
Cli = __decorate([
    inversify_1.injectable(),
    __metadata("design:paramtypes", [Object, Boolean])
], Cli);
exports.default = Cli;
;
//# sourceMappingURL=Cli.js.map