"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Exception_1 = require("../error/Exception");
class ModulesRegistry {
    constructor(context = {}) {
        this.context = context;
        this.modules = new Map();
        this.lazyModules = new Map();
    }
    get size() {
        return this.modules.size;
    }
    addLazyModules(modules) {
        modules.forEach((module) => this.setLazyModule(module.id, module));
    }
    removeLazyModule(moduleId) {
        this.lazyModules.delete(moduleId);
    }
    addModules(modules) {
        modules.forEach((module) => this.setModule(module.id, module));
    }
    setModule(moduleId, module) {
        this.removeLazyModule(moduleId);
        this.modules.set(moduleId, module);
    }
    setLazyModule(id, module) {
        this.lazyModules.set(id, module);
    }
    getModules() {
        return Array.from(this.modules.values()).sort((a, b) => {
            if (a.id == ".")
                return -1;
            if (b.id == ".")
                return 1;
            return a.id.split("/") < b.id.split("/") ? -1 : 1;
        });
    }
    getLazyModules() {
        return Array.from(this.lazyModules.values()).sort((a, b) => {
            if (a.id == ".")
                return -1;
            if (b.id == ".")
                return 1;
            return a.id.split("/") < b.id.split("/") ? -1 : 1;
        });
    }
    getModuleIds() {
        return this.getModules().map((module) => module.id);
    }
    hasLazyModule(id) {
        return this.lazyModules.has(id);
    }
    hasModule(id) {
        return this.modules.has(id);
    }
    getModule(id) {
        if (!this.hasModule(id)) {
            throw new Exception_1.default(`Module '${id}' is not registered in registry!`, { ModuleId: id });
        }
        return this.modules.get(id);
    }
    getLazyModule(id) {
        if (!this.hasLazyModule(id)) {
            throw new Exception_1.default(`Module '${id}' is not registered in registry!`, { ModuleId: id });
        }
        return this.lazyModules.get(id);
    }
    forEach(callback) {
        this.getModules().forEach(callback);
    }
}
exports.default = ModulesRegistry;
//# sourceMappingURL=ModulesRegistry.js.map