"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const Exception_1 = require("../error/Exception");
const tmp = require("tmp");
const fs = require("fs");
let TaskExecutionService = class TaskExecutionService {
    constructor() {
        this.$executors = new Map();
    }
    async executeTask(task, context, targetModule, bindingModule) {
        if (!task.metadata.executor) {
            throw new Exception_1.default(`Task has no executor defined!`, {
                TaskName: task.name,
                OwnerModuleId: targetModule.id
            });
        }
        const taskExecutor = this.getExecutor(task.metadata.executor);
        const opsContextFile = tmp.fileSync();
        fs.writeFileSync(opsContextFile.name, JSON.stringify({
            module: targetModule.serialize(),
            binding: bindingModule ? bindingModule.serializeAsBinding() : {}
        }));
        try {
            await taskExecutor.execute(task, {
                env: {
                    OPS_CONTEXT_FILE: opsContextFile.name,
                    OPS_TARGET_MODULE: module.id,
                    OPS_BINDING_MODULE: bindingModule ? bindingModule.id : undefined,
                    OPS_PATH_MODULES: context.modulesRoot
                }
            }, targetModule, bindingModule);
        }
        finally {
            opsContextFile.removeCallback();
        }
    }
    registerExecutor(instance) {
        this.$executors.set(instance.name, instance);
    }
    hasExecutor(name) {
        return this.$executors.has(name);
    }
    getExecutor(name) {
        if (!this.hasExecutor(name))
            throw new Exception_1.default(`Task executor is not found!`, {
                TaskName: name
            });
        return this.$executors.get(name);
    }
};
TaskExecutionService = __decorate([
    inversify_1.injectable()
], TaskExecutionService);
exports.default = TaskExecutionService;
//# sourceMappingURL=TaskExecutionService.js.map