"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const glob = require("glob");
const fs = require("fs");
const inversify_1 = require("inversify");
const Cli_1 = require("../../../cli/Cli");
const FSTaskConfigurationActions_1 = require("./FSTaskConfigurationActions");
const child_process_1 = require("child_process");
const YamlService_1 = require("../../../yaml/YamlService");
let FSTaskExecutor = class FSTaskExecutor {
    constructor($cli, $yaml) {
        this.$cli = $cli;
        this.$yaml = $yaml;
        this.name = "fs";
    }
    async execute(task, context, module, binding) {
        if (!task.configuration) {
            throw new Error;
        }
        this.$cli.logTrace("Executing task with configuration:", task.configuration);
        switch (task.configuration["action"]) {
            case FSTaskConfigurationActions_1.default.REMOVE:
                this.removeTargets(this.findTargets(task.configuration["selectors"], {
                    sync: true,
                    absolute: true,
                    realpath: true,
                    cwd: module.paths.directory
                }));
                break;
            case FSTaskConfigurationActions_1.default.WRITE:
                if (!task.configuration["content"] && task.configuration["content-from-exec"]) {
                    const execConfig = task.configuration["content-from-exec"];
                    task.configuration["content"] = child_process_1.execFileSync(execConfig.command, execConfig.args, {
                        cwd: execConfig.cwd || process.cwd(),
                        timeout: execConfig.timeout || undefined,
                        env: execConfig.env || process.env
                    }).toString("utf-8");
                }
                this.writeContentToTargets(this.normalizeTargets(task.configuration["selectors"], module.paths.directory), task.configuration["content"], task.configuration["content-type"]);
                break;
            default:
                throw new Error;
        }
    }
    normalizeTargets(selectors, cwd) {
        if (typeof selectors == "string") {
            selectors = [selectors];
        }
        return selectors;
    }
    findTargets(selectors, options) {
        if (typeof selectors == "string") {
            selectors = [selectors];
        }
        const targets = new Set();
        selectors.forEach((selector) => glob.sync(selector, options).forEach((target) => targets.add(target)));
        return Array.from(targets);
    }
    removeTargets(targets) {
        if (!targets.length) {
            this.$cli.logDebug("No files to remove!");
        }
        else {
            this.$cli.logTrace("Removing files:", targets);
            let removalFiles = 0;
            let removalDirectories = 0;
            targets.forEach((target) => {
                if (fs.lstatSync(target).isDirectory()) {
                    [
                        glob.sync("*", {
                            sync: true,
                            absolute: true,
                            realpath: true,
                            cwd: target
                        }),
                        glob.sync(".*", {
                            sync: true,
                            absolute: true,
                            realpath: true,
                            cwd: target
                        })
                    ].forEach((files) => {
                        if (files.length) {
                            const quiet = this.$cli.quiet;
                            this.$cli.quiet = true;
                            this.removeTargets(files);
                            this.$cli.quiet = quiet;
                        }
                    });
                    fs.rmdirSync(target);
                    removalDirectories++;
                }
                else {
                    fs.unlinkSync(target);
                    removalFiles++;
                }
            });
            this.$cli.logInfo(`Removed ${removalFiles} files!`);
            if (removalDirectories) {
                this.$cli.logInfo(`Removed ${removalDirectories} directories!`);
            }
        }
    }
    writeContentToTargets(targets, content, contentType) {
        if (typeof content != "string") {
            switch (contentType) {
                case "yaml":
                    content = this.$yaml.stringify(content);
                    break;
                default:
                    throw new Error;
            }
        }
        content = content.toString();
        targets.forEach((target) => {
            this.$cli.logDebug(`Writing (${contentType}) content (${content.length} bytes) into: ${target}`);
            fs.writeFileSync(target, content);
        });
    }
};
FSTaskExecutor = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(Cli_1.default)),
    __param(1, inversify_1.inject(YamlService_1.default)),
    __metadata("design:paramtypes", [Cli_1.default,
        YamlService_1.default])
], FSTaskExecutor);
exports.default = FSTaskExecutor;
//# sourceMappingURL=FSTaskExecutor.js.map