"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const TaskExecutionError_1 = require("../../TaskExecutionError");
const _ = require("lodash");
const OpsTaskConfiguration_1 = require("./OpsTaskConfiguration");
const Cli_1 = require("../../../cli/Cli");
const inversify_1 = require("inversify");
const TaskExecutionService_1 = require("../../TaskExecutionService");
const Models = require("../../../../domain/Models");
const ModulesService_1 = require("../../../modules/ModulesService");
let OpsTaskExecutor = class OpsTaskExecutor {
    constructor($cli, $modulesService, $tasksService) {
        this.$cli = $cli;
        this.$modulesService = $modulesService;
        this.$tasksService = $tasksService;
        this.name = "ops";
        this.configuration = {
            selector: {},
            execution: {
                type: "task",
                targets: [],
                ignoreErrors: false
            },
            registry: "ops",
            quiet: false,
            order: "regular",
            context: {}
        };
    }
    async execute(task, context, module, binding) {
        if (!task.configuration) {
            throw new Error;
        }
        const configuration = this.configuration = new OpsTaskConfiguration_1.default(task.configuration["selector"], _.merge({}, {
            ignoreErrors: false
        }, task.configuration["execution"] || {}), _.merge({}, task.configuration["context"] || {}, {
            binding: binding ? binding.serializeAsBinding() : module.serializeAsBinding()
        }), task.configuration["order"] || "regular", task.configuration["quiet"] || false, task.configuration["registry"] || "ops");
        if (!this.$modulesService.hasRegistry(configuration.registry)) {
            this.$modulesService.createRegistry(configuration.registry, configuration.context);
        }
        const selector = _.merge({}, { registry: configuration.registry }, configuration.selector);
        let targetModules = this.$modulesService.queryModules(selector, configuration.registry);
        targetModules.sort((a, b) => {
            // TODO: Add more selectors
            if (selector.pattern) {
                const patterns = selector.pattern instanceof Array ? selector.pattern : [selector.pattern];
                const aIndex = patterns.indexOf(a);
                const bIndex = patterns.indexOf(b);
                return aIndex < bIndex ? -1 : 1;
            }
            return 1;
        });
        if (!targetModules.length) {
            this.$cli.logWarn(`No module found for execution! Selector:`, selector);
        }
        if (configuration.execution.continueFrom) {
            const cutoff = targetModules.indexOf(configuration.execution.continueFrom);
            if (cutoff != -1) {
                this.$cli.logWarn(`Continuing execution from:`, configuration.execution.continueFrom);
                targetModules = targetModules.splice(cutoff);
            }
        }
        if (configuration.order == "reverse") {
            targetModules = targetModules.reverse();
        }
        if (configuration.execution.type == "lifecycle") {
            if (!configuration.execution.targets.length)
                throw new TaskExecutionError_1.default("No execution lifecycle targets specified!");
            await this.executeLifecycles(configuration.execution.targets, targetModules, binding || module, configuration.execution.ignoreErrors);
        }
        else if (configuration.execution.type == "task") {
            if (!configuration.execution.targets.length)
                throw new TaskExecutionError_1.default("No execution tasks specified!");
            await this.executeTasks(configuration.execution.targets, targetModules, binding || module, configuration.execution.ignoreErrors);
        }
        else {
            throw new TaskExecutionError_1.default("Invalid execution type specified!");
        }
    }
    resolveLifecycle(module, lifecycleName) {
        if (!module.lifecycles.has(lifecycleName))
            throw new TaskExecutionError_1.default(`Lifecycle '${lifecycleName}' in not found in module '${module.id}'!`);
        return module.lifecycles.get(lifecycleName);
    }
    resolveTask(module, taskName) {
        if (!module.tasks.has(taskName))
            throw new TaskExecutionError_1.default(`Task '${taskName}' in not found in module '${module.id}'!`);
        return module.tasks.get(taskName);
    }
    resolveLifecycleAction(module, lifecycleName, actionName) {
        const lifecycle = this.resolveLifecycle(module, lifecycleName);
        if (!lifecycle.actions.has(actionName))
            throw new TaskExecutionError_1.default(`Action '${actionName}' in not found in lifecycle '${lifecycle.name}' module '${module.id}'!`);
        return lifecycle.actions.get(actionName);
    }
    async executeLifecycles(lifecycles, modules, binding, ignoreErrors = false) {
        this.$cli.logTrace(`Executing lifecycles:`, {
            lifecycles: lifecycles,
            modules: modules
        });
        let executions = 0;
        for (let lifecycleIndex = 0; lifecycleIndex < lifecycles.length; lifecycleIndex++) {
            for (let moduleIndex = 0; moduleIndex < modules.length; moduleIndex++) {
                const moduleId = modules[moduleIndex];
                const module = await this.$modulesService.getModule(moduleId, this.configuration.registry);
                const lifecycleName = lifecycles[lifecycleIndex];
                if (!module.lifecycles.has(lifecycleName)) {
                    throw new Error(`Lifecycle '${lifecycleName}' is not defined in module '${module.id}'!`);
                }
                this.$cli.logModuleProcessingStart(`[${moduleIndex + 1}/${modules.length}] ${module.id}`);
                const lifecycle = module.lifecycles.get(lifecycleName);
                try {
                    if (lifecycle.dependencies) {
                        for (let dependencyIndex = 0; dependencyIndex < lifecycle.dependencies.length; dependencyIndex++) {
                            const dependencyLifecycleName = lifecycle.dependencies[dependencyIndex];
                            if (!module.lifecycles.has(dependencyLifecycleName)) {
                                throw new Error(`Lifecycle '${dependencyLifecycleName}' is not defined (dependency of ${lifecycleName})!`);
                            }
                            const dependencyLifecycle = module.lifecycles.get(dependencyLifecycleName);
                            if (!this.isLifecycleExecutable(dependencyLifecycle, module, binding)) {
                                continue;
                            }
                            this.$cli.logLifecycleStart(`${dependencyLifecycleName} (dependency)`);
                            try {
                                await this.executeLifecycle(dependencyLifecycle, module, binding);
                            }
                            catch (error) {
                                if (ignoreErrors) {
                                    this.$cli.logWarn(`Failure during module lifecycle execution (IGNORED):
Target: ${module.id} –> lifecycle:${lifecycle.name} –> lifecycle:${dependencyLifecycle} (dependency)
\n` + error.stack);
                                }
                                else {
                                    this.$cli.logLifecycleEnd();
                                    throw error;
                                }
                            }
                            this.$cli.logLifecycleEnd();
                        }
                    }
                    try {
                        this.$cli.logLifecycleStart(lifecycle.name);
                        await this.executeLifecycle(lifecycle, module, binding);
                    }
                    catch (error) {
                        if (ignoreErrors) {
                            this.$cli.logWarn(`Failure during module lifecycle execution (IGNORED):
Target: ${module.id} –> lifecycle:${lifecycle.name}
\n` + error.stack);
                        }
                        else {
                            this.$cli.logLifecycleEnd();
                            throw error;
                        }
                    }
                    this.$cli.logLifecycleEnd();
                    executions++;
                }
                catch (error) {
                    throw new TaskExecutionError_1.default(`Failure during module lifecycle execution:
Target: ${module.id} –> lifecycle:${lifecycle.name}
\n` + error.stack);
                }
            }
        }
        if (!executions)
            this.$cli.logWarn("No modules were processed!");
    }
    async executeTasks(tasks, modules, binding, ignoreErrors = false) {
        if (tasks.length) {
            this.$cli.logTrace(`Execution task:\n\t:${tasks.join("\n\t:")}${ignoreErrors ? "\n(ignore errors mode)" : ""}`);
        }
        let executions = 0;
        for (let taskIndex = 0; taskIndex < tasks.length; taskIndex++) {
            for (let moduleIndex = 0; moduleIndex < modules.length; moduleIndex++) {
                const moduleId = modules[moduleIndex];
                const module = await this.$modulesService.getModule(moduleId, this.configuration.registry);
                const taskName = tasks[taskIndex];
                if (!module.tasks.has(taskName)) {
                    this.$cli.logWarn(`Task '${taskName}' is not defined in module '${module.id}'!`);
                    continue;
                }
                const task = module.tasks.get(taskName);
                if (task.metadata.enabled === false) {
                    this.$cli.logDebug(`Task ${taskName} is disabled in module '${module.id}'!`);
                    continue;
                }
                this.$cli.logTaskStart(taskName);
                if (module.metadata.virtual && !task.metadata.executor) {
                    continue;
                }
                try {
                    try {
                        await this.executeTask(task, module, binding);
                    }
                    catch (error) {
                        if (ignoreErrors) {
                            this.$cli.logWarn(`Failure during module task execution (IGNORED):
Target: ${module.id} –> task:${task.name}
\n` + error.stack);
                        }
                        else {
                            this.$cli.logTaskEnd();
                            throw error;
                        }
                    }
                    this.$cli.logTaskEnd();
                    executions++;
                }
                catch (error) {
                    throw new TaskExecutionError_1.default(`Failure during module task execution:
Target: ${module.id} –> task:${task.name}
\n` + error.stack);
                }
            }
        }
        if (!executions) {
            this.$cli.logWarn("No modules were processed!");
        }
    }
    async executeLifecycleAction(lifecycle, action, module, binding) {
        const task = this.resolveTask(module, action.task.name);
        if (task.metadata.enabled === false) {
            this.$cli.logDebug(`Task '${task.name}' is disabled in module '${module.id}'!`);
            return;
        }
        if (module.metadata.virtual && !task.metadata.supportsVirtual) {
            throw new Error(`Task '${task.name}' doesn't support virtual modules!\nModule: ${module.id}`);
        }
        if (!module.metadata.enabled) {
            this.$cli.logDebug(`Module '${module.id}' is disabled!`);
            return;
        }
        if (!task.metadata.executor)
            throw new Error(`Task '${task.name}' has not executor defined!`);
        const lifecycleActionTask = new Models.ModuleTask(task.name, task.metadata.clone(), _.merge({}, task.configuration, action.task.configuration));
        try {
            await this.$tasksService.executeTask(lifecycleActionTask, global.application.$configuration.context, module, binding);
        }
        catch (error) {
            throw new TaskExecutionError_1.default(`Task execution failed!`, {
                Target: `${module.id} –> lifecycle:${lifecycle.name} ${action.name != action.task.name ? `–> (action:${action.name}) –> (task:${action.task.name})` : ""} –> (task:${action.task.name})`,
                Executor: task.metadata.executor,
                Configuration: lifecycleActionTask.configuration
            }, error);
        }
    }
    isLifecycleExecutable(lifecycle, module, binding) {
        const actions = new Array()
            .concat(lifecycle.actionsBefore.getAll())
            .concat(lifecycle.actions.getAll())
            .concat(lifecycle.actionsAfter.getAll());
        if (!actions.length)
            return false;
        for (let index = 0; index < actions.length; index++) {
            const action = actions[index];
            if (!module.tasks.has(action.task.name))
                throw new TaskExecutionError_1.default(`Task '${action.task.name}' in not found in module '${module.id}'!`);
            const task = module.tasks.get(action.task.name);
            if (task.metadata.enabled === false) {
                continue;
            }
            return true;
        }
    }
    async executeLifecycle(lifecycle, module, binding) {
        const actions = new Array()
            .concat(lifecycle.actionsBefore.getAll())
            .concat(lifecycle.actions.getAll())
            .concat(lifecycle.actionsAfter.getAll());
        if (!actions.length) {
            if (!lifecycle.dependencies) {
                this.$cli.logWarn(`Lifecycle '${lifecycle.name}' have no actions in module '${module.id}'!`);
            }
            return;
        }
        this.$cli.logTrace(`Execution flow for module:${module.id} task:${lifecycle.name}\n\t—> ` + actions.map((action) => action.name).join("\n\t—> "));
        if (!actions.length) {
            this.$cli.logWarn(`No actions to perform in ${module.id} lifecycle:${lifecycle.name}!`);
        }
        for (let index = 0; index < actions.length; index++) {
            const action = actions[index];
            if (!module.tasks.has(action.task.name))
                throw new TaskExecutionError_1.default(`Task '${action.task.name}' in not found in module '${module.id}'!`);
            const task = module.tasks.get(action.task.name);
            if (task.metadata.enabled === false) {
                this.$cli.logDebug(`Task '${action.name}' is disabled in module '${module.id}'!`);
                continue;
            }
            this.$cli.logTaskStart(action.name);
            await this.executeLifecycleAction(lifecycle, action, module, binding);
            this.$cli.logTaskEnd();
        }
    }
    async executeTask(task, targetModule, bindingModule) {
        if (targetModule.metadata.virtual && !task.metadata.supportsVirtual)
            throw new Error(`Task '${task.name}' doesn't support virtual modules!`);
        if (!task.configuration)
            throw new Error(`Task '${task.name}' has no default configuration!`);
        if (!task.metadata.executor)
            throw new Error(`Task '${task.name}' has no executor name!`);
        if (!targetModule.metadata.enabled) {
            this.$cli.logDebug(`Module '${targetModule.id}' is disabled!`);
            return;
        }
        try {
            await this.$tasksService.executeTask(task, global.application.$configuration.context, targetModule, bindingModule);
        }
        catch (error) {
            throw new TaskExecutionError_1.default(`Task execution failed!`, {
                Target: `${targetModule.id} –> task:${task.name}`,
                Executor: task.metadata.executor,
                Configuration: task.configuration
            }, error);
        }
    }
};
OpsTaskExecutor = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(Cli_1.default)),
    __param(1, inversify_1.inject(ModulesService_1.default)),
    __param(2, inversify_1.inject(TaskExecutionService_1.default)),
    __metadata("design:paramtypes", [Cli_1.default,
        ModulesService_1.default,
        TaskExecutionService_1.default])
], OpsTaskExecutor);
exports.default = OpsTaskExecutor;
//# sourceMappingURL=OpsTaskExecutor.js.map