"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const inversify_1 = require("inversify");
const Cli_1 = require("../../../cli/Cli");
const TerraformRemoteStatesGeneratorTaskConfiguration_1 = require("./TerraformRemoteStatesGeneratorTaskConfiguration");
let TerraformRemoteStatesGeneratorTaskExecutor = class TerraformRemoteStatesGeneratorTaskExecutor {
    constructor($cli) {
        this.$cli = $cli;
        this.name = "terraform-remote-states-generator";
    }
    serializeValue(value) {
        if (value instanceof Array) {
            return JSON.stringify(value.map((value) => {
                return this.serializeValue(value);
            }));
        }
        else if (value instanceof Object) {
            return "{" + Object.keys(value).map((key) => {
                return `${key} = ${this.serializeValue(value[key])}`;
            }).join("\n") + "}";
        }
        else {
            return JSON.stringify(value);
        }
    }
    async execute(task, context, module, binding) {
        const configuration = TerraformRemoteStatesGeneratorTaskConfiguration_1.default.parse(task.configuration, module, binding);
        this.$cli.logNamedYamlObject(configuration, `Task '${task.name}' configuration`);
        if (!configuration.remotes.length) {
            this.$cli.logDebug("No remotes specified!");
            return;
        }
        const cwd = module.paths.directory;
        const source = configuration.remotes.map((remote) => {
            return `
data "terraform_remote_state" "${remote.name}" {
  backend = "s3"
  defaults = ${this.serializeValue(remote.defaults)}
  config {
    key            = "${remote.directory}/${remote.state}.tfstate"
    bucket         = "${configuration.stateBucketName}"
    region         = "${configuration.stateRegion}"
    encrypt        = "${configuration.stateBucketEncrypt ? "true" : "false"}"
    dynamodb_table = "${configuration.stateDynamoDBTable}"
    access_key     = "${configuration.stateCredentialsAccessKey}"
    secret_key     = "${configuration.stateCredentialsSecretKey}"
  }
}
`;
        }).join("\n\n");
        fs.writeFileSync(`${cwd}/${configuration.outputFileName}`, source);
    }
};
TerraformRemoteStatesGeneratorTaskExecutor = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(Cli_1.default)),
    __metadata("design:paramtypes", [Cli_1.default])
], TerraformRemoteStatesGeneratorTaskExecutor);
exports.default = TerraformRemoteStatesGeneratorTaskExecutor;
//# sourceMappingURL=TerraformRemoteStatesGeneratorTaskExecutor.js.map