"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const yaml = require("yaml");
const child_process_1 = require("child_process");
const _ = require("lodash");
const TaskExecutionError_1 = require("../../TaskExecutionError");
const inversify_1 = require("inversify");
const Cli_1 = require("../../../cli/Cli");
const TerraformSectionSourcesGeneratorTaskConfiguration_1 = require("./TerraformSectionSourcesGeneratorTaskConfiguration");
const YamlService_1 = require("../../../yaml/YamlService");
const YamlLazyValue_1 = require("../../../yaml/YamlLazyValue");
let TerraformSectionSourcesGeneratorTaskExecutor = class TerraformSectionSourcesGeneratorTaskExecutor {
    constructor($cli, $yaml) {
        this.$cli = $cli;
        this.$yaml = $yaml;
        this.name = "terraform-section-sources-generator";
    }
    async execute(task, context, module, binding) {
        if (!task.configuration)
            throw new Error;
        const configuration = TerraformSectionSourcesGeneratorTaskConfiguration_1.default.parse(task.configuration);
        this.$cli.logNamedYamlObject(configuration, `Task '${task.name}' configuration`);
        if (!module.templates.has(configuration.templateSection)) {
            if (task.metadata.enabled) {
                throw new TaskExecutionError_1.default(`No template '${configuration.templateSection}' defined! ` +
                    `Found: ${JSON.stringify(module.templates.getNames())}!`);
            }
            else {
                this.$cli.logDebug(`No template '${configuration.templateSection}' defined! Skipping execution!`);
                return;
            }
        }
        const cwd = module.paths.directory;
        const inputFile = `${cwd}/~~terraform.sources.input.yaml`;
        let templateSection = module.templates.get(configuration.templateSection);
        if (typeof templateSection != "string") {
            templateSection = this.$yaml.stringify(templateSection);
        }
        fs.writeFileSync(inputFile, templateSection);
        const variablesData = {
            module: module.serialize()
        };
        const variablesStructure = {
            module: "(( &temporary ))"
        };
        if (binding) {
            variablesData.binding = binding.serialize();
            variablesStructure.binding = "(( &temporary ))";
        }
        const variablesYaml = this.$yaml.stringify(variablesData);
        const variablesFile = `${cwd}/~~terraform.sources.variables.yaml`;
        const templateData = _.merge(yaml.parse(fs.readFileSync(inputFile).toString()), variablesStructure);
        const templateYaml = this.$yaml.stringify(templateData);
        const templateFile = `${cwd}/~~terraform.sources.template.yaml`;
        this.$cli.logDebug(`Interpolating ${module.id} ${configuration.templateSection}:`);
        fs.writeFileSync(templateFile, templateYaml);
        fs.writeFileSync(variablesFile, variablesYaml);
        const resultStructure = this.$yaml.parseSpiffFiles([templateFile, variablesFile], {
            env: module.env.filter((value) => !(value instanceof YamlLazyValue_1.default)).serialize(),
            cwd: cwd
        });
        const resultJSON = JSON.stringify(resultStructure, null, 4);
        const resultFile = `${cwd}/~~terraform.source.result.yaml`;
        this.$cli.logDebug(`Generating ${module.id} ${configuration.templateSection}:`);
        fs.writeFileSync(resultFile, resultJSON);
        const outputHCL = child_process_1.execFileSync("bash", ['-c', `json2hcl < ${resultFile}`], {
            cwd: cwd,
            env: _.merge({}, process.env, module.env.toHash())
        })
            .toString()
            .split("\n")
            .map((line) => line.replace(/^(\s*)?"([a-z_-]*)"/g, "$1$2"))
            .join("\n")
            .replace(/\n\n/g, "\n");
        fs.writeFileSync(`${cwd}/${configuration.outputFileName}`, outputHCL);
        this.$cli.logDebug(`Generated terraform sources '${configuration.outputFileName}'`);
    }
};
TerraformSectionSourcesGeneratorTaskExecutor = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(Cli_1.default)),
    __param(1, inversify_1.inject(YamlService_1.default)),
    __metadata("design:paramtypes", [Cli_1.default,
        YamlService_1.default])
], TerraformSectionSourcesGeneratorTaskExecutor);
exports.default = TerraformSectionSourcesGeneratorTaskExecutor;
//# sourceMappingURL=TerraformSectionSourcesGeneratorTaskExecutor.js.map