"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const _ = require("lodash");
const glob = require("glob");
const TerraformSourcesImporterTaskConfiguration_1 = require("./TerraformSourcesImporterTaskConfiguration");
const inversify_1 = require("inversify");
const Cli_1 = require("../../../cli/Cli");
const TaskExecutionError_1 = require("../../TaskExecutionError");
const ModulesService_1 = require("../../../modules/ModulesService");
let TerraformSourcesImporterTaskExecutor = class TerraformSourcesImporterTaskExecutor {
    constructor($cli, $modulesService) {
        this.$cli = $cli;
        this.$modulesService = $modulesService;
        this.name = "terraform-sources-importer";
    }
    async execute(task, context, module, binding) {
        const configuration = TerraformSourcesImporterTaskConfiguration_1.default.parse(task.configuration);
        this.$cli.logNamedYamlObject(configuration, `Task '${task.name}' configuration`);
        const cwd = module.paths.directory;
        const importedTerraformSources = [];
        const importedTerraformVariables = {};
        this.$modulesService.createRegistry("task", {
            binding: binding.serializeAsBinding()
        });
        const modules = this.$modulesService.queryModules(configuration.selector, "task");
        this.$cli.logDebug("Importing terraform sources from modules: ", modules);
        for (let moduleIndex = 0; moduleIndex < modules.length; moduleIndex++) {
            const moduleId = modules[moduleIndex];
            const module = await this.$modulesService.getModule(moduleId, "task");
            const sourceFiles = glob.sync("*.tf", {
                cwd: module.paths.directory,
                absolute: true
            });
            sourceFiles.forEach((file) => {
                this.$cli.logDebug(`- ${file}`);
                importedTerraformSources.push(`/* ${file} */`, fs.readFileSync(file).toString());
            });
            const jsonVariableFiles = glob.sync("*.tfvars.json", {
                cwd: module.paths.directory,
                absolute: true
            });
            jsonVariableFiles.forEach((file) => {
                this.$cli.logDebug(`- ${file}`);
                _.merge(importedTerraformVariables, JSON.parse(fs.readFileSync(file).toString()));
            });
        }
        fs.writeFileSync(`${cwd}/${configuration.outputFilePrefix}.tf`, importedTerraformSources.join("\n\n"));
        if (Object.keys(importedTerraformVariables).length) {
            fs.writeFileSync(`${cwd}/${configuration.outputFilePrefix}.tfvars.json`, JSON.stringify(importedTerraformVariables, null, 4));
        }
        if (modules.length) {
            this.$cli.logInfo(`Imported terraform sources from ${modules.length} modules!`);
        }
        else {
            throw new TaskExecutionError_1.default(`No modules for terraform sources import!`, {
                Configuration: task.configuration
            });
        }
    }
};
TerraformSourcesImporterTaskExecutor = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(Cli_1.default)),
    __param(1, inversify_1.inject(ModulesService_1.default)),
    __metadata("design:paramtypes", [Cli_1.default,
        ModulesService_1.default])
], TerraformSourcesImporterTaskExecutor);
exports.default = TerraformSourcesImporterTaskExecutor;
//# sourceMappingURL=TerraformSourcesImporterTaskExecutor.js.map