"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Exception_1 = require("../../../error/Exception");
const _ = require("lodash");
const Validator = require("fastest-validator");
class TerraformTaskMoveStateFileConfiguration {
    constructor(stateFileTarget, stateFileSource, stateFileMethod, stateBucketName, stateRegion, stateCredentialsAccessKey, stateCredentialsSecretKey) {
        this.stateFileTarget = stateFileTarget;
        this.stateFileSource = stateFileSource;
        this.stateFileMethod = stateFileMethod;
        this.stateBucketName = stateBucketName;
        this.stateRegion = stateRegion;
        this.stateCredentialsAccessKey = stateCredentialsAccessKey;
        this.stateCredentialsSecretKey = stateCredentialsSecretKey;
    }
    static parse(configuration, module, binding) {
        if (!configuration)
            throw new Error("Missing configuration!");
        configuration = _.merge({
            state: {
                file: {
                    method: "copy",
                    target: module.context.environment.terraform.state.file
                },
                bucket: {
                    name: module.context.environment.terraform.backend.s3.bucket.name,
                    region: module.context.environment.terraform.backend.s3.region,
                    credentials: {
                        "access-key": module.context.environment.terraform.backend.s3.credentials["access-key"],
                        "secret-key": module.context.environment.terraform.backend.s3.credentials["secret-key"]
                    }
                }
            }
        }, configuration);
        const validator = new Validator();
        const errors = validator.validate(configuration, {
            state: {
                type: "object",
                props: {
                    file: {
                        type: "object",
                        props: {
                            target: { type: "string" },
                            source: { type: "string" },
                            method: { type: "string" },
                        }
                    },
                    bucket: {
                        type: "object",
                        props: {
                            name: { type: "string" },
                            region: { type: "string" },
                            credentials: {
                                type: "object",
                                props: {
                                    "access-key": { type: "string" },
                                    "secret-key": { type: "string" }
                                }
                            }
                        }
                    }
                }
            }
        });
        if (errors !== true)
            throw new Exception_1.default("Invalid task configuration:", errors);
        return new TerraformTaskMoveStateFileConfiguration(configuration.state.file.target, configuration.state.file.source, configuration.state.file.method, configuration.state.bucket.name, configuration.state.bucket.region, configuration.state.bucket.credentials["access-key"], configuration.state.bucket.credentials["secret-key"]);
    }
}
exports.default = TerraformTaskMoveStateFileConfiguration;
//# sourceMappingURL=TerraformTaskMoveStateFileConfiguration.js.map