"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const Exception_1 = require("../../../error/Exception");
const Validator = require("fastest-validator");
class TerraformTaskMoveStateResourcesConfiguration {
    constructor(stateResourcesMove, stateFile, stateDirectory, stateBucketName, stateRegion, stateCredentialsAccessKey, stateCredentialsSecretKey) {
        this.stateResourcesMove = stateResourcesMove;
        this.stateFile = stateFile;
        this.stateDirectory = stateDirectory;
        this.stateBucketName = stateBucketName;
        this.stateRegion = stateRegion;
        this.stateCredentialsAccessKey = stateCredentialsAccessKey;
        this.stateCredentialsSecretKey = stateCredentialsSecretKey;
    }
    static parse(configuration, module, binding) {
        if (!configuration)
            throw new Error("Missing configuration!");
        configuration = _.merge({
            state: {
                file: module.context.environment.terraform.state.id + ".tfstate",
                directory: module.context.environment.terraform.state.directory,
                bucket: {
                    name: module.context.environment.terraform.backend.s3.bucket.name,
                    region: module.context.environment.terraform.backend.s3.region,
                    credentials: {
                        "access-key": module.context.environment.terraform.backend.s3.credentials["access-key"],
                        "secret-key": module.context.environment.terraform.backend.s3.credentials["secret-key"]
                    }
                }
            }
        }, configuration);
        const validator = new Validator();
        const errors = validator.validate(configuration, {
            state: {
                type: "object",
                props: {
                    file: { type: "string" },
                    directory: { type: "string" },
                    bucket: {
                        type: "object",
                        props: {
                            name: { type: "string" },
                            region: { type: "string" },
                            credentials: {
                                type: "object",
                                props: {
                                    "access-key": { type: "string" },
                                    "secret-key": { type: "string" }
                                }
                            }
                        }
                    }
                }
            }
        });
        if (errors !== true)
            throw new Exception_1.default("Invalid task configuration:", errors);
        const stateResourcesMove = new Map();
        Object.keys(configuration.state.resources.move).forEach((oldPath) => {
            const newPath = configuration.state.resources.move[oldPath];
            stateResourcesMove.set(oldPath, newPath);
        });
        return new TerraformTaskMoveStateResourcesConfiguration(stateResourcesMove, configuration.state.file, configuration.state.directory, configuration.state.bucket.name, configuration.state.bucket.region, configuration.state.bucket.credentials["access-key"], configuration.state.bucket.credentials["secret-key"]);
    }
}
exports.default = TerraformTaskMoveStateResourcesConfiguration;
//# sourceMappingURL=TerraformTaskMoveStateResourcesConfiguration.js.map