"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const Cli_1 = require("../../../cli/Cli");
const glob = require("glob");
const _ = require("lodash");
const fs = require("fs");
const ejs = require("ejs");
const Paths = require("path");
const YamlService_1 = require("../../../yaml/YamlService");
const YamlLazyValue_1 = require("../../../yaml/YamlLazyValue");
let TerraformTemplateSourcesGeneratorTaskExecutor = class TerraformTemplateSourcesGeneratorTaskExecutor {
    constructor($cli, $yaml) {
        this.$cli = $cli;
        this.$yaml = $yaml;
        this.name = "terraform-template-sources-generator";
    }
    async execute(task, context, module, binding) {
        const cwd = module.paths.directory;
        const spiffFiles = glob.sync("*.tftemplate", {
            cwd: cwd
        });
        for (let index = 0; index < spiffFiles.length; index++) {
            const file = spiffFiles[index];
            await this.generateSpiffTemplate(fs.readFileSync(`${cwd}/${file}`).toString(), Paths.basename(file, ".tftemplate"), module, binding);
        }
        if (spiffFiles.length) {
            this.$cli.logInfo(`Generated ${spiffFiles.length} terraform files from spiff template!`);
        }
        const jsFiles = glob.sync("*.tfjs", {
            cwd: cwd
        });
        for (let index = 0; index < jsFiles.length; index++) {
            const file = jsFiles[index];
            await this.generateJsTemplate(fs.readFileSync(`${cwd}/${file}`).toString(), Paths.basename(file, ".tfjs"), module, binding);
        }
        if (jsFiles.length) {
            this.$cli.logInfo(`Generated ${jsFiles.length} terraform files from js template!`);
        }
        const ejsFiles = glob.sync("*.tfejs", {
            cwd: cwd
        });
        for (let index = 0; index < ejsFiles.length; index++) {
            const file = ejsFiles[index];
            await this.generateEjsTemplate(fs.readFileSync(`${cwd}/${file}`).toString(), Paths.basename(file, ".tfejs"), module, binding);
        }
        if (ejsFiles.length) {
            this.$cli.logInfo(`Generated ${ejsFiles.length} terraform files from ejs template!`);
        }
    }
    async generateJsTemplate(source, name, module, binding) {
        this.$cli.logDebug(`Interpolating ${module.id} ${name}:`);
        const cwd = module.paths.directory;
        const output = [];
        new Function("output", "module", "binding", source)(output, module.serialize(), binding.serialize());
        fs.writeFileSync(`${cwd}/~${name}.tf`, output.join("\n"));
        this.$cli.logDebug(`Generated ${cwd}/~${name}.tf terraform file from js template!`);
    }
    async generateEjsTemplate(source, name, module, binding) {
        this.$cli.logDebug(`Interpolating ${module.id} ${name}:`);
        const cwd = module.paths.directory;
        const output = ejs.render(source, {
            module: module.serialize(),
            binding: binding.serializeAsBinding(),
            require: require,
            application: global['application']
        });
        fs.writeFileSync(`${cwd}/~${name}.tf`, output);
        this.$cli.logDebug(`Generated ${cwd}/~${name}.tf terraform file from js template!`);
    }
    async generateSpiffTemplate(source, name, module, binding) {
        this.$cli.logDebug(`Interpolating ${module.id} ${name}:`);
        const cwd = module.paths.directory;
        const spiffExpressions = new Array()
            .concat(source.match(/(?<=<<SPIFF)(.*?)(?=SPIFF)/gmus) || [])
            .concat(source.match(/(?<=<<\()(.*?)(?=\)>>)/gmus) || []);
        if (!spiffExpressions.length) {
            return;
        }
        const inputStructure = {};
        spiffExpressions.forEach((expression, index) => {
            inputStructure["exp" + index] = expression.trim();
        });
        const inputSource = this.$yaml.stringify(inputStructure);
        const inputFile = `${cwd}/~~${name}.input.yaml`;
        fs.writeFileSync(inputFile, inputSource);
        const variablesData = {
            module: module.serialize()
        };
        const variablesStructure = {
            module: "(( &temporary ))"
        };
        if (binding) {
            variablesData.binding = binding.serialize();
            variablesStructure.binding = "(( &temporary ))";
        }
        const variablesYaml = this.$yaml.stringify(variablesData);
        const variablesFile = `${cwd}/~~${name}.variables.yaml`;
        const templateData = _.merge(this.$yaml.parseFile(inputFile), variablesStructure);
        const templateYaml = this.$yaml.stringify(templateData);
        const templateFile = `${cwd}/~~${name}.template.yaml`;
        fs.writeFileSync(templateFile, templateYaml);
        fs.writeFileSync(variablesFile, variablesYaml);
        const resultYaml = this.$yaml.spiffFiles([templateFile, variablesFile], {
            env: module.env.filter((value) => !(value instanceof YamlLazyValue_1.default)).serialize(),
            cwd: cwd
        });
        const resultStructure = this.$yaml.parse(resultYaml, {
            env: module.env.filter((value) => !(value instanceof YamlLazyValue_1.default)).serialize(),
            cwd: cwd
        });
        const resultFile = `${cwd}/~~${name}.result.yaml`;
        fs.writeFileSync(resultFile, resultYaml);
        spiffExpressions.forEach((expression, index) => {
            source = source.replace("<<SPIFF" + expression + "SPIFF\n", JSON.stringify(typeof resultStructure["exp" + index] == "string" ?
                resultStructure["exp" + index].trim() :
                resultStructure["exp" + index]));
            source = source.replace("<<(" + expression + ")>>", resultStructure["exp" + index]);
        });
        fs.writeFileSync(`${cwd}/~${name}.tf`, source);
        this.$cli.logDebug(`Generated ${cwd}/~${name}.tf terraform file from spiff template!`);
    }
};
TerraformTemplateSourcesGeneratorTaskExecutor = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(Cli_1.default)),
    __param(1, inversify_1.inject(YamlService_1.default)),
    __metadata("design:paramtypes", [Cli_1.default,
        YamlService_1.default])
], TerraformTemplateSourcesGeneratorTaskExecutor);
exports.default = TerraformTemplateSourcesGeneratorTaskExecutor;
//# sourceMappingURL=TerraformTemplateSourcesGeneratorTaskExecutor.js.map