"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const _ = require("lodash");
const flat = require("flat");
const TerraformVariablesGeneratorTaskConfiguration_1 = require("./TerraformVariablesGeneratorTaskConfiguration");
const inversify_1 = require("inversify");
const Cli_1 = require("../../../cli/Cli");
const TaskExecutionError_1 = require("../../TaskExecutionError");
const YamlService_1 = require("../../../yaml/YamlService");
const YamlLazyValue_1 = require("../../../yaml/YamlLazyValue");
const glob = require("glob");
let TerraformVariablesGeneratorTaskExecutor = class TerraformVariablesGeneratorTaskExecutor {
    constructor($cli, $yaml) {
        this.$cli = $cli;
        this.$yaml = $yaml;
        this.name = "terraform-variables-generator";
    }
    async execute(task, context, module, binding) {
        if (!task.configuration)
            throw new Error;
        const configuration = new TerraformVariablesGeneratorTaskConfiguration_1.default(task.configuration["output-file-prefix"], task.configuration["template-section"], task.configuration["definitions-create"], task.configuration["values-format"], task.configuration["defined-only"], task.configuration["context"]);
        const cwd = module.paths.directory;
        let inputFile = `${cwd}/terraform.tfvars.yaml`;
        if (!fs.existsSync(inputFile) && module.templates.has(configuration.templateSection)) {
            inputFile = `${cwd}/~~terraform.variables.input.yaml`;
            let template = module.templates.get(configuration.templateSection);
            if (typeof template != "string") {
                template = this.$yaml.stringify(template);
            }
            let inputStructure = this.$yaml.parse(template, {
                cwd
            });
            if (inputStructure instanceof Array) {
                inputStructure = _.merge({}, ...inputStructure);
            }
            fs.writeFileSync(inputFile, this.$yaml.stringify(inputStructure));
        }
        else {
            if (task.metadata.enabled) {
                throw new TaskExecutionError_1.default(`No template '${configuration.templateSection}' defined! Found: ${JSON.stringify(module.templates.getNames())}!`);
            }
            else {
                this.$cli.logDebug(`No terraform.tfvars.yaml file found or '${configuration.templateSection}' template defined!`);
                return;
            }
        }
        const contextStructure = {
            module: module.serialize(),
            binding: binding ? binding.serializeAsBinding() : undefined
        };
        if (!contextStructure.binding) {
            delete contextStructure.binding;
        }
        const inputStructure = this.$yaml.parseFile(inputFile);
        const templateStructure = {
            variables: inputStructure,
            module: "(( &temporary ))",
            binding: "(( &temporary ))"
        };
        const templateYaml = this.$yaml.stringify(templateStructure);
        const templateFile = `${cwd}/~~terraform.variables.template.yaml`;
        const contextYaml = this.$yaml.stringify(contextStructure);
        const contextFile = `${cwd}/~~terraform.variables.context.yaml`;
        this.$cli.logDebug(`Interpolating ${module.id} ${configuration.templateSection}:`);
        fs.writeFileSync(templateFile, templateYaml);
        fs.writeFileSync(contextFile, contextYaml);
        const outputYaml = this.$yaml.spiffFiles([templateFile, contextFile], {
            env: module.env.filter((value) => !(value instanceof YamlLazyValue_1.default)).serialize(),
            cwd: cwd
        });
        const outputStructure = this.$yaml.parse(outputYaml, {
            env: module.env.filter((value) => !(value instanceof YamlLazyValue_1.default)).serialize(),
            cwd: cwd
        });
        const variablesStructure = flat(outputStructure.variables, { delimiter: "-", safe: true });
        const maxDepth = Object.keys(variablesStructure)
            .map((s) => s.split("-").length)
            .reduce((p, c, i, a) => Math.max((p || 0), c));
        for (let depth = 1; depth < (maxDepth - 1); depth++) {
            const depthStructure = flat(outputStructure.variables, { delimiter: "-", safe: true, maxDepth: depth });
            for (let key in depthStructure) {
                if (typeof variablesStructure[key] == "undefined") {
                    variablesStructure[key] = depthStructure[key];
                }
            }
        }
        fs.writeFileSync(`${cwd}/~~terraform.variables.result.yaml`, outputYaml);
        if (configuration.valuesFormat == TerraformVariablesGeneratorTaskConfiguration_1.ValueFormat.JSON) {
            const variablesJson = JSON.stringify(variablesStructure, null, 4);
            fs.writeFileSync(`${cwd}/${configuration.outputFilePrefix}.tfvars.json`, variablesJson);
        }
        this.$cli.logNamedYaml(outputYaml, `Interpolation module:${module.id} manifest:${configuration.templateSection}:`);
        const variablesFilter = [];
        if (configuration.definedOnly) {
            glob.sync("*.tf", {
                sync: true,
                absolute: true,
                realpath: true,
                cwd: cwd
            }).forEach((file) => {
                const content = fs.readFileSync(file).toString();
                (content.match(/var\.([a-zA-Z0-9_-]+)/gm) || []).forEach((definition) => variablesFilter.push(definition.substr(4, definition.length - 4)));
            });
        }
        if (configuration.definitionsCreate) {
            const definitionsCode = Object
                .keys(variablesStructure)
                .map((variable) => {
                if (configuration.definedOnly && !variablesFilter.includes(variable)) {
                    return "";
                }
                const name = variable.replace(/([.\/])/g, "-");
                const type = (variablesStructure[variable] instanceof Array ? "list" :
                    variablesStructure[variable] instanceof Object ? "map" : "string");
                const defaults = (function () {
                    if (configuration.valuesFormat != TerraformVariablesGeneratorTaskConfiguration_1.ValueFormat.DEFAULTS) {
                        return;
                    }
                    // Consider better replacer for objects
                    if (type != "string") {
                        return JSON.stringify(variablesStructure[variable], null, 4).replace(/": /g, "\" = ");
                    }
                    else {
                        return JSON.stringify(variablesStructure[variable], null, 4);
                    }
                }());
                if (defaults == "null")
                    return "";
                const buffer = [`variable "${name}" {`, `\ttype = "${type}"`];
                if (defaults != undefined)
                    buffer.push(`\tdefault = ${defaults}`);
                buffer.push(`}`);
                return buffer.join("\n");
            })
                .filter((string) => string.length)
                .join("\n");
            fs.writeFileSync(`${cwd}/${configuration.outputFilePrefix}.tf`, definitionsCode);
            this.$cli.logInfo(`Generated terraform variables '${configuration.outputFilePrefix}.tf'`);
        }
    }
};
TerraformVariablesGeneratorTaskExecutor = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(Cli_1.default)),
    __param(1, inversify_1.inject(YamlService_1.default)),
    __metadata("design:paramtypes", [Cli_1.default,
        YamlService_1.default])
], TerraformVariablesGeneratorTaskExecutor);
exports.default = TerraformVariablesGeneratorTaskExecutor;
//# sourceMappingURL=TerraformVariablesGeneratorTaskExecutor.js.map