"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const TerragruntConfigurationGeneratorTaskConfiguration_1 = require("./TerragruntConfigurationGeneratorTaskConfiguration");
const child_process_1 = require("child_process");
const _ = require("lodash");
const fs = require("fs");
const inversify_1 = require("inversify");
const Cli_1 = require("../../../cli/Cli");
const YamlService_1 = require("../../../yaml/YamlService");
const YamlLazyValue_1 = require("../../../yaml/YamlLazyValue");
let TerragruntConfigurationGeneratorTaskExecutor = class TerragruntConfigurationGeneratorTaskExecutor {
    constructor($cli, $yaml) {
        this.$cli = $cli;
        this.$yaml = $yaml;
        this.name = "terragrunt-configuration-generator";
    }
    async execute(task, context, module, binding) {
        const configuration = TerragruntConfigurationGeneratorTaskConfiguration_1.default.parse(task.configuration, module, binding);
        this.$cli.logNamedYamlObject(configuration, `Task '${task.name}' configuration`);
        this.$cli.logDebug(`Interpolating ${module.id} ${configuration.templateSection}:`);
        const cwd = module.paths.directory;
        const inputHclFile = `${cwd}/~~terragrunt.configuration.input.hcl`;
        const inputJsonFile = `${cwd}/~~terragrunt.configuration.input.json`;
        const inputYamlFile = `${cwd}/~~terragrunt.configuration.input.yaml`;
        const inputHcl = module.templates.get(configuration.templateSection);
        fs.writeFileSync(inputHclFile, inputHcl);
        const inputJson = child_process_1.execFileSync("bash", ['-c', `json2hcl -reverse < ${inputHclFile}`], {
            cwd: cwd,
            env: _.merge({}, process.env, module.env.toHash())
        }).toString();
        fs.writeFileSync(inputJsonFile, inputJson);
        const inputStructure = JSON.parse(inputJson);
        const inputYaml = this.$yaml.stringify(inputStructure);
        fs.writeFileSync(inputYamlFile, inputYaml);
        //
        const variablesData = {
            module: module.serialize()
        };
        const variableStructure = {
            module: "(( &temporary ))"
        };
        if (binding) {
            variablesData.binding = binding.serialize();
            variableStructure.binding = "(( &temporary ))";
        }
        const templateYaml = this.$yaml.stringify(_.merge(variableStructure, inputStructure));
        const templateFile = `${cwd}/~~terragrunt.configuration.template.yaml`;
        fs.writeFileSync(templateFile, templateYaml);
        const variables = this.$yaml.stringify(variablesData);
        const variablesFile = `${cwd}/~~terragrunt.configuration.variables.yaml`;
        fs.writeFileSync(variablesFile, variables);
        try {
            const resultStructure = this.$yaml.parseSpiffFiles([templateFile, variablesFile], {
                env: module.env.filter((value) => !(value instanceof YamlLazyValue_1.default)).serialize(),
                cwd: cwd
            });
            const resultJson = JSON.stringify(resultStructure, null, 4);
            const jsonFile = `${cwd}/~~terragrunt.configuration.result.json`;
            fs.writeFileSync(jsonFile, resultJson);
            const resultHcl = child_process_1.execFileSync("bash", ['-c', `json2hcl < ${jsonFile}`], {
                cwd: cwd,
                env: _.merge({}, process.env, module.env.toHash())
            }).toString()
                .replace(/\n\n/g, "\n")
                .split("\n")
                .map((line) => line.replace(/^(\s*)?"([a-z_-]*)"/g, "$1$2"))
                .join("\n");
            fs.writeFileSync(`${cwd}/${configuration.outputFileName}`, resultHcl);
            this.$cli.logInfo(`Generated terragrunt configuration '${configuration.outputFileName}'`);
        }
        catch (error) {
            this.$cli.logNamedYamlObject(variablesData, "Context", true);
            throw error;
        }
    }
};
TerragruntConfigurationGeneratorTaskExecutor = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(Cli_1.default)),
    __param(1, inversify_1.inject(YamlService_1.default)),
    __metadata("design:paramtypes", [Cli_1.default,
        YamlService_1.default])
], TerragruntConfigurationGeneratorTaskExecutor);
exports.default = TerragruntConfigurationGeneratorTaskExecutor;
//# sourceMappingURL=TerragruntConfigurationGeneratorTaskExecutor.js.map