"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const Path = require("path");
const fs = require("fs");
const Exception_1 = require("../error/Exception");
const yaml = require("yaml");
const inversify_1 = require("inversify");
const child_process_1 = require("child_process");
const _ = require("lodash");
const YamlLazyValue_1 = require("./YamlLazyValue");
const CronParser = require("cron-parser");
const Moment = require("moment");
let YamlService = class YamlService {
    constructor() {
    }
    parseJsonFile(path, context = {}) {
        path = fs.realpathSync(path);
        if (!fs.existsSync(path)) {
            throw `Module manifest path ${path} doesn't exists!`;
        }
        if (!fs.lstatSync(path).isFile()) {
            throw `Module manifest path ${path} is not a file!`;
        }
        const source = fs.readFileSync(path).toString("utf-8");
        return JSON.parse(source);
    }
    parseFile(path, context = {}) {
        path = fs.realpathSync(path);
        if (!fs.existsSync(path)) {
            throw `Module manifest path ${path} doesn't exists!`;
        }
        if (!fs.lstatSync(path).isFile()) {
            throw `Module manifest path ${path} is not a file!`;
        }
        const source = fs.readFileSync(path).toString("utf-8");
        return this.parse(source, _.merge({
            path: path
        }, context));
    }
    stringify(source, context = {}) {
        try {
            return JSON.stringify(source, null, 2);
        }
        catch (error) {
            throw new Exception_1.default("Unable to stringify yaml", {
                Source: source,
                Path: context.path
            }, error);
        }
    }
    parseSpiffFiles(paths, context = {}) {
        return this.parse(this.spiffFiles(paths, context), _.merge({
            path: "spiff " + paths.join(" ")
        }, context));
    }
    spiffFiles(paths, context = {}) {
        const options = {
            cwd: context.cwd || Path.dirname(paths[0]),
            env: _.merge({}, process.env, context.env),
            encoding: "utf-8",
            timeout: 60000
        };
        const args = ["m"];
        if (context.asJson) {
            args.push("--json");
        }
        args.push(...paths);
        global.application.$cli.logTrace("Spiffing yaml files...", {
            command: "spiff++",
            args: args,
            options: options
        });
        return child_process_1.execFileSync("spiff++", args, options).toString("utf-8");
    }
    spiffStream(content, context = {}) {
        const options = {
            cwd: context.cwd,
            env: _.merge({}, process.env, context.env),
            encoding: "utf-8",
            timeout: 60000,
            input: content
        };
        global.application.$cli.logTrace("Spiffing yaml stream...", {
            command: "spiff++",
            args: ["m", "-"],
            options: options
        });
        return child_process_1.execFileSync("spiff++", ["m", "-"], options).toString("utf-8");
    }
    parse(source, context = {}) {
        try {
            source = source
                .replace(/<<: !/g, "<<-: !")
                .replace(/"<<": !/g, "<<-: !");
            return yaml.parse(source, {
                merge: false,
                keepNodeTypes: true,
                keepCstNodes: true,
                customTags: this.createTags(context)
            });
        }
        catch (error) {
            throw new Exception_1.default("Unable to parse yaml", {
                Source: source,
                Path: context.path || ""
            }, error);
        }
    }
    createTags(context) {
        if (context.path && !context.cwd) {
            context.cwd = Path.dirname(context.path);
        }
        return [
            {
                class: YamlLazyValue_1.default,
                default: false,
                tag: '!lazy',
                resolve: (document, node) => {
                    const value = node.context.src.substr(node.valueRange.start, node.valueRange.end - node.valueRange.start);
                    if (!value.length) {
                        throw new Exception_1.default("Invalid !lazy value");
                    }
                    return new YamlLazyValue_1.default(value);
                },
                stringify: (item) => {
                    return item.value.value.toString();
                }
            },
            {
                class: Function,
                default: false,
                tag: '!js',
                resolve: (document, node) => {
                    const body = node.context.src.substr(node.valueRange.start, node.valueRange.end - node.valueRange.start);
                    try {
                        // @ts-ignore
                        global['context'] = context;
                        return eval("(" + body.trim() + ")");
                    }
                    catch (error) {
                        throw new Exception_1.default("Invalid !js tag: " + error.message, {
                            Source: body
                        }, error);
                    }
                },
                stringify: (item) => {
                    return JSON.stringify(item.value.toString());
                }
            },
            {
                class: Function,
                default: false,
                tag: '!include.js',
                resolve: (document, node) => {
                    const body = node.context.src.substr(node.valueRange.start, node.valueRange.end - node.valueRange.start);
                    try {
                        return require(context.cwd + "/" + body.trim())(context || {});
                    }
                    catch (error) {
                        throw new Exception_1.default("Invalid !include.js tag: " + error.message, {
                            Source: body
                        }, error);
                    }
                },
                stringify: (item) => {
                    return JSON.stringify(item.value.toString());
                }
            },
            {
                class: String,
                default: false,
                tag: '!direct-inheritor',
                resolve: (document, node) => {
                    return '!direct-inheritor ' + node.context.src.substr(node.valueRange.start, node.valueRange.end - node.valueRange.start);
                },
                stringify: () => {
                }
            },
            {
                class: String,
                default: false,
                tag: '!path.resolve',
                resolve: (document, node) => {
                    if (!context.cwd) {
                        throw new Exception_1.default("No current directory context!");
                    }
                    return Path.resolve(context.cwd, node.context.src.substr(node.valueRange.start, node.valueRange.end - node.valueRange.start));
                },
                stringify: () => {
                }
            },
            {
                class: String,
                default: false,
                tag: '!include.file',
                resolve: (document, node) => {
                    if (!context.cwd) {
                        throw new Exception_1.default("No current directory context!");
                    }
                    const includePath = Path.resolve(context.cwd, node.context.src.substr(node.valueRange.start, node.valueRange.end - node.valueRange.start));
                    return fs.readFileSync(includePath.trim()).toString();
                },
                stringify: () => {
                }
            },
            {
                class: String,
                default: false,
                tag: '!include.yaml',
                resolve: (document, node) => {
                    if (!context.cwd) {
                        throw new Exception_1.default("No current directory context!");
                    }
                    const includePath = Path.resolve(context.cwd, node.context.src.substr(node.valueRange.start, node.valueRange.end - node.valueRange.start)).trim();
                    return this.parseFile(includePath.trim());
                },
                stringify: () => {
                }
            },
            {
                class: String,
                default: false,
                tag: '!env',
                resolve: (document, node) => {
                    const name = node.context.src.substr(node.valueRange.start, node.valueRange.end - node.valueRange.start);
                    const variable = name.split(" ", 1)[0];
                    const defaults = name.split(" ").slice(1).join(" ");
                    const value = process.env.hasOwnProperty(variable) ?
                        process.env[variable] :
                        defaults;
                    if (typeof value == "undefined") {
                        throw new Exception_1.default(`Environment variable is not found!`, {
                            Name: name,
                            Defaults: defaults,
                            Env: process.env[variable]
                        });
                    }
                    return value;
                },
                stringify: () => {
                }
            },
            {
                class: String,
                default: false,
                tag: '!cron.next',
                resolve: (document, node) => {
                    const pattern = node.context.src.substr(node.valueRange.start, node.valueRange.end - node.valueRange.start);
                    return CronParser.parseExpression(JSON.parse(pattern), { utc: true }).next().toISOString();
                },
                stringify: (item) => {
                    return JSON.stringify(item.value.toString());
                }
            },
            {
                class: String,
                default: false,
                tag: '!moment.add',
                resolve: (document, node) => {
                    const pattern = node.context.src.substr(node.valueRange.start, node.valueRange.end - node.valueRange.start);
                    return Moment().add(JSON.parse(pattern)).toISOString();
                },
                stringify: (item) => {
                    return JSON.stringify(item.value.toString());
                }
            },
            {
                class: String,
                default: false,
                tag: '!moment.add-start-of-day',
                resolve: (document, node) => {
                    const pattern = node.context.src.substr(node.valueRange.start, node.valueRange.end - node.valueRange.start);
                    return Moment().add(JSON.parse(pattern)).startOf('day').toISOString();
                },
                stringify: (item) => {
                    return JSON.stringify(item.value.toString());
                }
            }
        ];
    }
};
YamlService = __decorate([
    inversify_1.injectable(),
    __metadata("design:paramtypes", [])
], YamlService);
exports.default = YamlService;
//# sourceMappingURL=YamlService.js.map