"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const _ = require("lodash");
const Models = require("./Models");
const SchemaValidator_1 = require("./SchemaValidator");
const ModuleMetadataInheritanceInto_1 = require("./models/ModuleMetadataInheritanceInto");
const YamlService_1 = require("../components/yaml/YamlService");
const packageJson = require("./../../package.json");
let ModelsBuilder = class ModelsBuilder {
    constructor($validator, $yaml) {
        this.$validator = $validator;
        this.$yaml = $yaml;
    }
    buildModule(manifest) {
        this.$validator.validateManifest(manifest);
        const module = manifest.module;
        if (!module) {
            throw new SyntaxError(`Missing .module definition!`);
        }
        this.$validator.validateModule(module);
        const moduleMetadata = this.parseModuleMetadata(module, module.metadata);
        const moduleInputParameters = this.parseModuleInputParameters(module, module.inputs);
        const modulePaths = this.parseModulePaths(module, module.paths);
        const moduleEnv = this.parseModuleEnv(module, module.env);
        const moduleContext = this.parseModuleContext(module, module.context);
        const moduleLifecycles = this.parseModuleLifecycles(module, module.lifecycles);
        const moduleTasks = this.parseModuleTasks(module, module.tasks);
        const moduleTemplates = this.parseModuleTemplates(module, module.templates, moduleMetadata);
        const moduleHelpers = this.parseModuleHelpers(module, module.helpers);
        return new Models.Module(module.id, moduleMetadata, moduleInputParameters, modulePaths, moduleEnv, moduleLifecycles, moduleTasks, moduleTemplates, moduleHelpers, moduleContext);
    }
    parseModuleMetadata(module, metadata = {}) {
        if (!metadata) {
            throw new SyntaxError(`Missing .module.metadata section!`);
        }
        return new Models.ModuleMetadata(metadata.enabled == undefined ? true : metadata.enabled, metadata.virtual == undefined ? false : metadata.virtual, metadata.binding == undefined ? false : metadata.binding, this.parseModuleMetadataInheritance(module, metadata.inherit), this.parseModuleMetadataTags(module, metadata.tags), metadata.schema == undefined ? packageJson.version : metadata.schema, this.parseModuleMetadataAutoload(module, metadata.autoload), metadata.$interpolation || "none", metadata.$parents == undefined ? new Set() : new Set(metadata.$parents));
    }
    parseModuleMetadataInheritance(module, inherit) {
        switch (true) {
            case !inherit:
                return new Models.ModuleMetadataInheritance();
            case inherit === true:
                return new Models.ModuleMetadataInheritance(new ModuleMetadataInheritanceInto_1.default({
                    child: true
                }));
            case typeof inherit == "object":
                inherit = inherit;
                const inheritInto = !inherit.into ?
                    new ModuleMetadataInheritanceInto_1.default() :
                    new ModuleMetadataInheritanceInto_1.default(inherit.into);
                return new Models.ModuleMetadataInheritance(inheritInto);
            default:
                throw new SyntaxError("Invalid .module.metadata.inherit section! " +
                    JSON.stringify(inherit));
        }
    }
    parseModuleMetadataTags(module, tags) {
        const result = new Set();
        if (tags) {
            tags.forEach((tag) => result.add(tag));
        }
        return result;
    }
    parseModuleMetadataAutoload(module, patterns) {
        const result = new Set();
        if (patterns) {
            patterns.forEach((pattern) => result.add(pattern));
        }
        return result;
    }
    parseModulePaths(module, paths) {
        if (!paths)
            throw new SyntaxError("Missing .module.paths section!");
        if (!paths.manifest)
            throw new SyntaxError("Missing .module.paths.manifest field!");
        if (!paths.module)
            throw new SyntaxError("Missing .module.paths.module field!");
        if (!paths.directory)
            throw new SyntaxError("Missing .module.paths.directory field!");
        if (!paths.unit)
            throw new SyntaxError("Missing .module.paths.unit field!");
        if (!paths.root)
            throw new SyntaxError("Missing .module.paths.root field!");
        return new Models.ModulePaths(paths.manifest, paths.module, paths.directory, paths.unit, paths.root);
    }
    parseModuleEnv(module, env) {
        if (!env) {
            return new Models.ModuleEnv();
        }
        if (typeof env == "object") {
            const map = new Map();
            Object.keys(env).forEach((key) => map.set(key, env[key]));
            return new Models.ModuleEnv(map);
        }
        throw new SyntaxError("Invalid .module.env section!");
    }
    parseModuleContext(module, context) {
        return context || {};
    }
    parseModuleLifecycles(module, lifecycles) {
        if (!lifecycles) {
            return new Models.ModuleLifecycles();
        }
        if (lifecycles instanceof Array) {
            return this.parseModuleLifecyclesArray(module, lifecycles);
        }
        if (lifecycles instanceof Object) {
            return this.parseModuleLifecyclesHash(module, lifecycles);
        }
        throw new SyntaxError("Invalid .module.lifecycles definition: not a object or array!");
    }
    parseModuleInputParameters(module, inputs) {
        if (!inputs) {
            return new Models.ModuleInputParameters();
        }
        if (inputs instanceof Object) {
            return this.parseModuleInputParametersHash(module, inputs);
        }
        throw new SyntaxError("Invalid .module.inputs definition: not a object!");
    }
    parseModuleHelpers(module, helpers) {
        if (!helpers) {
            return new Models.ModuleHelpers();
        }
        if (helpers instanceof Object) {
            return this.parseModuleHelpersHash(module, helpers);
        }
        throw new SyntaxError("Invalid .module.helpers definition: not a object!");
    }
    parseModuleTasks(module, tasks) {
        if (!tasks) {
            return new Models.ModuleTasks();
        }
        if (tasks instanceof Array) {
            return this.parseModuleTasksArray(module, tasks);
        }
        if (tasks instanceof Object) {
            return this.parseModuleTasksHash(module, tasks);
        }
        throw new SyntaxError("Invalid .module.tasks definition: not a object or array!");
    }
    parseModuleTemplates(module, templates, metadata) {
        const collection = new Map();
        if (templates) {
            Object.keys(templates).forEach((name) => {
                let template = templates[name];
                collection.set(name, template);
            });
        }
        return new Models.ModuleTemplates(collection);
    }
    parseModuleLifecyclesArray(module, lifecycles) {
        const collection = new Map();
        lifecycles.forEach((lifecycle, index) => {
            if (!lifecycle.name)
                throw new SyntaxError;
            collection.set(lifecycle.name, this.parseModuleLifecycle(module, lifecycle));
        });
        return new Models.ModuleLifecycles(collection);
    }
    parseModuleLifecyclesHash(module, lifecycles) {
        const collection = new Map();
        for (const name in lifecycles) {
            const lifecycle = lifecycles[name];
            if (!(lifecycle instanceof Object))
                throw new SyntaxError("Invalid lifecycle definition: not a object!");
            collection.set(name, this.parseModuleLifecycle(module, _.merge({}, lifecycles[name], { name: name })));
        }
        return new Models.ModuleLifecycles(collection);
    }
    parseModuleLifecycle(module, lifecycle) {
        if (!lifecycle.name) {
            throw new SyntaxError;
        }
        return new Models.ModuleLifecycle(lifecycle.name, lifecycle.dependencies || [], this.parseModuleLifecycleActions(module, lifecycle.actions), this.parseModuleLifecycleActions(module, lifecycle["actions-before"]), this.parseModuleLifecycleActions(module, lifecycle["actions-after"]));
    }
    parseModuleLifecycleActions(module, actions) {
        const collection = new Array();
        if (actions) {
            if (!(actions instanceof Array)) {
                throw new SyntaxError("Invalid lifecycle tasks definition: not array!");
            }
            collection.push(...actions.map((action) => this.parseModuleLifecycleAction(module, action)));
        }
        return new Models.ModuleLifecycleActions(collection);
    }
    parseModuleLifecycleAction(module, action) {
        if (typeof action == "string")
            return new Models.ModuleLifecycleAction(action, new Models.ModuleLifecycleActionTask(action, {}));
        if (action instanceof Object) {
            if (!action.task || typeof action.task == "string") {
                if (!action.name) {
                    throw new SyntaxError;
                }
                if (!action.task) {
                    throw new SyntaxError;
                }
                return new Models.ModuleLifecycleAction(action.name, this.parseModuleLifecycleActionTask(module, { name: action.task || action.name }));
            }
            else {
                if (!action.name)
                    throw new SyntaxError;
                return new Models.ModuleLifecycleAction(action.name, this.parseModuleLifecycleActionTask(module, action.task));
            }
        }
        else {
            throw new SyntaxError("Invalid lifecycle action definition!");
        }
    }
    parseModuleLifecycleActionTask(module, task) {
        if (!task.name) {
            throw new SyntaxError();
        }
        return new Models.ModuleLifecycleActionTask(task.name, task.configuration);
    }
    parseModuleTasksArray(module, tasks) {
        const collection = new Map();
        tasks.forEach((task) => {
            if (!task.name) {
                throw new SyntaxError("Missing task name!");
            }
            collection.set(task.name, this.parseModuleTask(module, task));
        });
        return new Models.ModuleTasks(collection);
    }
    parseModuleInputParametersHash(module, inputs) {
        const collection = new Map();
        for (const name in inputs) {
            const input = inputs[name];
            if (!(input instanceof Object)) {
                throw new SyntaxError("Invalid input definition: not a object!");
            }
            collection.set(name, this.parseModuleInputParameter(module, _.merge({}, inputs[name], { name: name })));
        }
        return new Models.ModuleInputParameters(collection);
    }
    parseModuleHelpersHash(module, helpers) {
        const collection = new Map();
        for (const name in helpers) {
            let helper = helpers[name];
            if (typeof helper == "string") {
                helper = eval("(" + helper + ")");
            }
            if (!(helper instanceof Function)) {
                throw new SyntaxError("Invalid helper definition: not a function!");
            }
            collection.set(name, helper);
        }
        return new Models.ModuleHelpers(collection);
    }
    parseModuleTasksHash(module, tasks) {
        const collection = new Map();
        for (const name in tasks) {
            const task = tasks[name];
            if (!(task instanceof Object)) {
                throw new SyntaxError("Invalid task definition: not a object!");
            }
            collection.set(name, this.parseModuleTask(module, _.merge({}, tasks[name], { name: name })));
        }
        return new Models.ModuleTasks(collection);
    }
    parseModuleTask(module, task) {
        if (!task.name)
            throw new SyntaxError("Missing task name!");
        //if (!task.executor) throw new SyntaxError(`Missing task '${task.name}' executor!`);
        return new Models.ModuleTask(task["name"], this.parseModuleTaskMetadata(module, task.metadata), task["configuration"]);
    }
    parseModuleInputParameter(module, parameter) {
        if (!parameter.name)
            throw new SyntaxError("Missing input parameter name!");
        return new Models.ModuleInputParameter(parameter["name"], parameter["required"] == undefined ? true : parameter["required"], parameter["provided"] == undefined ? false : parameter["provided"], parameter["value"], parameter["prompt"] == undefined ? undefined : this.parseModuleInputParameterPrompt(module, parameter, parameter["prompt"]));
    }
    parseModuleTaskMetadata(module, metadata) {
        return !metadata ?
            new Models.ModuleTaskMetadata() :
            new Models.ModuleTaskMetadata(metadata.enabled, metadata.executor, metadata["supports-virtual"], metadata.description);
    }
    parseModuleInputParameterPrompt(module, parameter, prompt) {
        if (!prompt) {
            return;
        }
        if (!prompt.message) {
            throw new SyntaxError("Missing prompt message!");
        }
        return new Models.ModuleInputParameterPrompt(prompt.message, prompt.type, prompt.choices, prompt.defaults);
    }
};
ModelsBuilder = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(SchemaValidator_1.default)),
    __param(1, inversify_1.inject(YamlService_1.default)),
    __metadata("design:paramtypes", [SchemaValidator_1.default,
        YamlService_1.default])
], ModelsBuilder);
exports.default = ModelsBuilder;
//# sourceMappingURL=ModelsBuilder.js.map