"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
class Module {
    constructor(id, metadata, inputs, paths, env, lifecycles, tasks, templates, helpers, context) {
        this.id = id;
        this.metadata = metadata;
        this.inputs = inputs;
        this.paths = paths;
        this.env = env;
        this.lifecycles = lifecycles;
        this.tasks = tasks;
        this.templates = templates;
        this.helpers = helpers;
        this.context = context;
    }
    clone() {
        return new Module(this.id, this.metadata.clone(), this.inputs.clone(), this.paths.clone(), this.env.clone(), this.lifecycles.clone(), this.tasks.clone(), this.templates.clone(), this.helpers.clone(), _.cloneDeep(this.context));
    }
    serialize() {
        const serialized = {
            id: this.id,
            metadata: this.metadata.serialize(),
            inputs: this.inputs.serialize(),
            paths: this.paths.serialize()
        };
        if (this.env.size)
            serialized.env = this.env.serialize();
        if (Object.keys(this.context).length)
            serialized.context = _.cloneDeep(this.context);
        if (this.lifecycles.size)
            serialized.lifecycles = this.lifecycles.serialize();
        if (this.tasks.size)
            serialized.tasks = this.tasks.serialize();
        if (this.templates.size)
            serialized.templates = this.templates.serialize();
        if (this.inputs.size)
            serialized.inputs = this.inputs.serialize();
        if (this.helpers.size)
            serialized.helpers = this.helpers.serialize();
        return serialized;
    }
    serializeAsBinding() {
        const serialized = {
            id: this.id,
            metadata: this.metadata.serialize(),
            inputs: this.inputs.serialize(),
            paths: this.paths.serialize()
        };
        if (this.env.size)
            serialized.env = this.env.serialize();
        if (Object.keys(this.context).length)
            serialized.context = _.cloneDeep(this.context);
        if (this.templates.size)
            serialized.templates = this.templates.serialize();
        if (this.inputs.size)
            serialized.inputs = this.inputs.serialize();
        if (this.helpers.size)
            serialized.helpers = this.helpers.serialize();
        return serialized;
    }
    inheritFrom(source) {
        if (this.metadata.parents.has(source.id))
            return;
        this.env.inheritFrom(source.env);
        this.inputs.inheritFrom(source.inputs);
        this.context = _.merge({}, source.context, this.context);
        this.lifecycles.inheritFrom(source.lifecycles);
        this.tasks.inheritFrom(source.tasks);
        this.templates.inheritFrom(source.templates);
        this.helpers.inheritFrom(source.helpers);
        this.metadata.inheritFrom(source.metadata);
        this.metadata.parents.add(source.id);
    }
}
exports.default = Module;
//# sourceMappingURL=Module.js.map