"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const YamlLazyValue_1 = require("../../components/yaml/YamlLazyValue");
class ModuleEnv {
    constructor(variables = new Map()) {
        this.variables = variables;
    }
    get size() {
        return this.variables.size;
    }
    clone() {
        return new ModuleEnv(new Map(Array.from(this.variables.entries())));
    }
    forEach(callback) {
        this.variables.forEach(callback);
    }
    get(name) {
        if (!this.has(name))
            throw new Error(`Environment variable ${name} does not exists!`);
        return this.variables.get(name);
    }
    has(key) {
        return this.variables.has(key);
    }
    inheritFrom(source) {
        source.forEach((value, name) => !this.has(name) && this.variables.set(name, value));
    }
    serialize() {
        const serialized = {};
        this.forEach((value, name) => {
            if (value instanceof YamlLazyValue_1.default) {
                serialized[name] = value.toString();
            }
            else {
                serialized[name] = value;
            }
        });
        return serialized;
    }
    toHash() {
        return this.serialize();
    }
    filter(filter) {
        const map = new Map();
        this.forEach((value, key) => {
            if (filter(value, key)) {
                map.set(key, value);
            }
        });
        return new ModuleEnv(map);
    }
}
exports.default = ModuleEnv;
//# sourceMappingURL=ModuleEnv.js.map