"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class ModuleInputParameters {
    constructor(inputs = new Map()) {
        this.inputs = inputs;
    }
    get size() {
        return this.inputs.size;
    }
    get entries() {
        return Array.from(this.inputs.entries());
    }
    clone() {
        return new ModuleInputParameters(new Map(Array.from(this.inputs.entries())));
    }
    forEach(callback) {
        this.inputs.forEach(callback);
    }
    get(name) {
        if (!this.has(name))
            throw new Error(`Input variable ${name} does not exists!`);
        return this.inputs.get(name);
    }
    has(key) {
        return this.inputs.has(key);
    }
    inheritFrom(source) {
        source.forEach((value, name) => !this.has(name) && this.inputs.set(name, value));
    }
    serialize() {
        const serialized = {};
        this.forEach((value, name) => serialized[name] = value.serialize());
        return serialized;
    }
    toHash() {
        return this.serialize();
    }
    filter(filter) {
        const map = new Map();
        this.forEach((value, key) => {
            if (filter(value, key)) {
                map.set(key, value);
            }
        });
        return new ModuleInputParameters(map);
    }
}
exports.default = ModuleInputParameters;
//# sourceMappingURL=ModuleInputParameters.js.map