"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class ModuleLifecycleActions {
    constructor(actions = new Array()) {
        this.actions = actions;
    }
    get size() {
        return this.actions.length;
    }
    getNames() {
        return this.actions.map((action) => action.name);
    }
    get(name) {
        if (!this.has(name))
            throw new Error(`Action '${name}' does not exists!`);
        return this.lookup(name);
    }
    has(name) {
        return this.lookup(name) != undefined;
    }
    clone() {
        const actions = new Array();
        this.forEach((action) => actions.push(action));
        return new ModuleLifecycleActions(actions);
    }
    forEach(callback) {
        this.actions.forEach(callback);
    }
    inheritFrom(actions, configuration = { prepend: false }) {
        const newActions = [];
        actions.forEach((action) => {
            if (this.has(action.name)) {
                const target = this.get(action.name);
                target.inheritFrom(action);
            }
            else {
                newActions.push(action.clone());
            }
        });
        if (configuration.prepend) {
            this.prepend(...newActions);
        }
        else {
            this.append(...newActions);
        }
    }
    serialize() {
        const serialized = [];
        this.forEach((action, index) => {
            const serializedAction = action.serialize();
            if (serializedAction.name && Object.keys(serializedAction).length == 1) {
                serialized[index] = serializedAction.name;
            }
            else {
                serialized[index] = serializedAction;
            }
        });
        return serialized;
    }
    getAll() {
        return Array.from(this.actions.values());
    }
    lookup(name) {
        return this.actions.find((action) => action.name == name);
    }
    prepend(...actions) {
        this.actions.unshift(...actions);
    }
    append(...action) {
        this.actions.push(...action);
    }
}
exports.default = ModuleLifecycleActions;
//# sourceMappingURL=ModuleLifecycleActions.js.map