"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class ModuleLifecycles {
    constructor(lifecycles = new Map()) {
        this.lifecycles = lifecycles;
    }
    get size() {
        return this.lifecycles.size;
    }
    get(name) {
        if (!this.has(name))
            throw new Error(`Lifecycle '${name}' does not exists!`);
        return this.lifecycles.get(name);
    }
    has(name) {
        return this.lifecycles.has(name);
    }
    getNames() {
        return Array.from(this.lifecycles.keys());
    }
    forEach(callback) {
        this.lifecycles.forEach(callback);
    }
    inheritFrom(lifecycles) {
        lifecycles.forEach((lifecycle) => {
            if (this.has(lifecycle.name)) {
                const target = this.get(lifecycle.name);
                target.inheritFrom(lifecycle);
            }
            else {
                this.lifecycles.set(lifecycle.name, lifecycle.clone());
            }
        });
    }
    clone() {
        const lifecycles = new Map();
        this.lifecycles.forEach((lifecycle) => lifecycles.set(lifecycle.name, lifecycle));
        return new ModuleLifecycles(lifecycles);
    }
    serialize() {
        const serialized = {};
        this.forEach((lifecycle) => {
            serialized[lifecycle.name] = lifecycle.serialize();
            delete serialized[lifecycle.name].name;
        });
        return serialized;
    }
    toHash() {
        const hash = {};
        if (this.lifecycles.size)
            this.lifecycles.forEach((lifecycle, name) => hash[name] = lifecycle);
        return hash;
    }
    filter(filter) {
        const map = new Map();
        this.forEach((value, key) => {
            if (filter(value, key)) {
                map.set(key, value);
            }
        });
        return new ModuleLifecycles(map);
    }
}
exports.default = ModuleLifecycles;
//# sourceMappingURL=ModuleLifecycles.js.map