"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class ModuleTasks {
    constructor(tasks = new Map()) {
        this.tasks = tasks;
    }
    get size() {
        return this.tasks.size;
    }
    get(name) {
        if (!this.has(name))
            throw new Error(`Task '${name}' does not exists!`);
        return this.tasks.get(name);
    }
    has(name) {
        return this.tasks.has(name);
    }
    forEach(callback) {
        this.tasks.forEach(callback);
    }
    inheritFrom(tasks) {
        tasks.forEach((task) => {
            if (this.has(task.name)) {
                this.get(task.name).inheritFrom(task);
            }
            else {
                this.addTask(task.clone());
            }
        });
    }
    clone() {
        const content = new Map();
        this.forEach((task) => content.set(task.name, task));
        return new ModuleTasks(content);
    }
    serialize() {
        const serialized = {};
        if (this.tasks.size)
            this.tasks.forEach((task, name) => serialized[name] = task.serialize());
        return serialized;
    }
    toHash() {
        const hash = {};
        if (this.tasks.size)
            this.tasks.forEach((task, name) => hash[name] = task);
        return hash;
    }
    addTask(task) {
        this.tasks.set(task.name, task.clone());
    }
    filter(filter) {
        const map = new Map();
        this.forEach((value, key) => {
            if (filter(value, key)) {
                map.set(key, value);
            }
        });
        return new ModuleTasks(map);
    }
}
exports.default = ModuleTasks;
//# sourceMappingURL=ModuleTasks.js.map