"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
class ModuleTemplates {
    constructor(templates = new Map()) {
        this.templates = templates;
    }
    get size() {
        return this.templates.size;
    }
    get(name) {
        if (!this.has(name))
            throw new Error(`Template '${name}' does not exists!`);
        return this.templates.get(name);
    }
    has(name) {
        return this.templates.has(name);
    }
    getNames() {
        return Array.from(this.templates.keys());
    }
    getAll() {
        return Array.from(this.templates.entries()).map((entry) => {
            return {
                name: entry[0],
                content: entry[1]
            };
        });
    }
    forEach(callback) {
        this.templates.forEach(callback);
    }
    inheritFrom(templates) {
        templates.forEach((content, name) => {
            if (this.has(name)) {
                const template = this.templates.get(name);
                if ((template instanceof Object) && (content instanceof Object)) {
                    this.templates.set(name, _.merge({}, template, content));
                }
            }
            else {
                this.templates.set(name, content);
            }
        });
    }
    clone() {
        const templates = new Map();
        this.forEach((content, name) => templates.set(name, content));
        return new ModuleTemplates(templates);
    }
    serialize() {
        const serialized = {};
        this.forEach((content, name) => serialized[name] = content);
        return serialized;
    }
    toHash() {
        return this.serialize();
    }
    filter(filter) {
        const map = new Map();
        this.forEach((value, key) => {
            if (filter(value, key)) {
                map.set(key, value);
            }
        });
        return new ModuleTemplates(map);
    }
}
exports.default = ModuleTemplates;
//# sourceMappingURL=ModuleTemplates.js.map