import * as parseCliArguments from "command-line-args";
import * as fs from "fs";
import * as Path from "path";
import applicationCliArguments from "./OpsApplicationCliArguments"
import {injectable} from "inversify";
import OpsApplicationContext from "./OpsApplicationContext";

@injectable()
export default class OpsApplicationContextReader {

    public readContext(): OpsApplicationContext {
        const args = parseCliArguments(applicationCliArguments, {camelCase: true});
        const modulesRoot = this.sanitizePathManifests(
            args.pathManifests
            || process.env.OPS_PATH_MODULES
            || ""
        );
        const bindingModule = this.sanitizeModule(
            args.binding
            || process.env.OPS_BINDING_MODULE
            || ""
        );
        const targetModule = this.sanitizeModule(
            args.module
            || process.env.OPS_TARGET_MODULE
            || (function () {
                const cwd = Path.normalize(process.cwd());
                if (cwd.startsWith(modulesRoot)) {
                    if (fs.existsSync(`${cwd}/module.yml`) || fs.existsSync(`${cwd}/module.yaml`)) {
                        return "." + cwd.substr(modulesRoot.length);
                    }
                }
            }())
            || ""
        );
        const targetLifecycle = this.sanitizeLifecycle(
            args.lifecycle
            || process.env.OPS_TARGET_LIFECYCLE
            || ""
        );
        const targetTask = this.sanitizeTask(
            args.task
            || process.env.OPS_TARGET_TASK
            || ""
        );
        const logLevel = args.logLevel
            || process.env.OPS_LOG_LEVEL
            || "info";
        const inputs = ((args.inputs || process.env.OPS_INPUTS || "").length ?
            JSON.parse(args.inputs || process.env.OPS_INPUTS) :
            {});
        const cmdTerraform = process.env.OPS_CMD_TERRAFORM || "terraform";
        const cmdTerragrunt = process.env.OPS_CMD_TERRAGRUNT || "terragrunt";
        const cmdSpiff = process.env.OPS_CMD_SPIFF || "spiff++";
        const cmdAWS = process.env.OPS_CMD_AWS || "aws";
        const continueFrom = args.continueFrom || "";
        return {
            tempDir: "",
            bindingModule,
            targetModule,
            targetLifecycle,
            targetTask,
            modulesRoot,
            logLevel,
            inputs,
            continueFrom,
            ignoreErrors: args.ignoreErrors || false,
            quiet: args.quiet || false,
            help: args.help || false,
            cmdTerraform: cmdTerraform,
            cmdTerragrunt: cmdTerragrunt,
            cmdSpiff: cmdSpiff,
            cmdAWS: cmdAWS
        };
    }

    private sanitizeModule(name: string): string {
        name = name.trim();
        ["/", "/module.yaml", "/module.yml"].forEach((postfix) => {
            if (name.toLowerCase().endsWith(postfix.toLowerCase())) {
                name = name.substr(0, name.length - postfix.length)
            }
        });
        if (!name.length) return name;
        if (!name.startsWith(".")) {
            name = "./" + name;
        }
        name = name.replace(/\/\//g, "/");
        return name;
    }

    private sanitizeLifecycle(name: string): string {
        return name.trim();
    }

    private sanitizeTask(name: string): string {
        return name.trim();
    }

    private sanitizePathManifests(path: string): string {
        return Path.normalize(path.trim());
    }

}
