import * as showCliUsage from "command-line-usage";
import {inject, injectable} from "inversify";
import Cli from "../components/cli/Cli";
import OpsApplicationCliArguments from "./OpsApplicationCliArguments";
import OpsApplicationContext from "./OpsApplicationContext";

@injectable()
export default class OpsApplicationUI {

    constructor(
        @inject(Cli)
        private readonly $cli: Cli,
        private packageJson = require("./../../package.json")
    ) {
    }

    showApplicationHelp() {
        this.$cli.log(showCliUsage([
            {
                header: `Ops (v${this.packageJson.version})`,
                content: 'Perform ops tasks'
            },
            {
                header: 'Arguments',
                optionList: OpsApplicationCliArguments
            }
        ]));
    }

    showApplicationInfo(context: OpsApplicationContext) {
        this.$cli.logHeader(`Ops (v${this.packageJson.version})`);
        context.targetModule && this.$cli.logHeader(`Module: ${context.targetModule}`);
        context.bindingModule && this.$cli.logHeader(`Binding: ${context.bindingModule}`);
        context.targetLifecycle && this.$cli.logHeader(`Lifecycle: ${context.targetLifecycle}`);
        context.targetTask && this.$cli.logHeader(`Task: ${context.targetTask}`);
    }
}
