import * as fs from "fs";
import * as glob from "glob";
import * as Path from "path";
import {inject, injectable} from "inversify";
import Cli from "../cli/Cli";

@injectable()
export default class ManifestsScanner {

    constructor(
        @inject(Cli) private $cli: Cli,
    ) {
    }

    find(location: string) {
        if (!fs.existsSync(location))
            throw `Modules directory path ${location} doesn't exists!`;
        if (!fs.lstatSync(location).isDirectory())
            throw `Modules directory path ${location} is not a directory!`;
        return glob
            .sync(
                `${location}/**/module.yaml`,
                {realpath: true}
            )
            .sort(this.sortByHierarchy);
    }

    private sortByHierarchy(a: string, b: string) {
        const pathsA = Path.dirname(a).split("/");
        const pathsB = Path.dirname(b).split("/");
        return pathsA < pathsB ? -1 : 1;
    }

}
