import ExecTaskConfiguration from "./ExecTaskConfiguration";
import {inject, injectable} from "inversify";
import TaskExecutor from "../../TaskExecutor";
import Cli from "../../../cli/Cli";
import * as Models from "../../../../domain/Models";
import {execFileSync} from "child_process";
import * as _ from "lodash";
import OpsApplicationConfiguration from "../../../../application/OpsApplicationConfiguration";
import TaskExecutorContext from "../../TaskExecutorContext";

@injectable()
export default class ExecTaskExecutor
    implements TaskExecutor {

    public readonly name: string = "exec";

    constructor(
        @inject(Cli)
        protected $cli: Cli,
        @inject(OpsApplicationConfiguration)
        protected $applicationConfiguration: OpsApplicationConfiguration
    ) {
    }

    async execute(
        task: Models.ModuleTask,
        context: TaskExecutorContext,
        module: Models.Module,
        binding?: Models.Module
    ) {
        if (!task.configuration) throw new Error;
        const configuration = new ExecTaskConfiguration(
            task.configuration["command"],
            task.configuration["args"] || [],
            _.merge(
                {
                    OPS_INPUTS: JSON.stringify(this.$applicationConfiguration.context.inputs),
                },
                context.env || {},
                process.env,
                module.env.serialize(),
                task.configuration["env"]
            ),
            task.configuration["cwd"] || module.paths.directory,
            task.configuration["timeout"]
        );
        this.$cli.logTrace("Executing: ", configuration);
        execFileSync(configuration.command, configuration.args, {
            cwd: configuration.cwd,
            timeout: configuration.timeout,
            env: configuration.env,
            stdio: "inherit"
        });
    }

}
