import TaskExecutor from "../../TaskExecutor";
import ManifestTaskConfiguration, {ManifestTaskConfigurationActions} from "./ManifestTaskConfiguration";
import Cli from "../../../cli/Cli";
import {inject, injectable} from "inversify";
import * as Models from "../../../../domain/Models";
import TaskExecutorContext from "../../TaskExecutorContext";

@injectable()
export default class ManifestTaskExecutor
    implements TaskExecutor {

    public readonly name: string = "manifest";

    constructor(
        @inject(Cli) protected $cli: Cli
    ) {
    }

    async execute(
        task: Models.ModuleTask,
        context: TaskExecutorContext,
        module: Models.Module,
        binding: Models.Module
    ) {
        if (!task.configuration) throw new Error;
        const configuration = new ManifestTaskConfiguration(
            task.configuration["action"]
        );
        if (configuration.action == ManifestTaskConfigurationActions.INSPECT) {
            const state: { binding?: any, module: any } = {
                module: module.serialize()
            };
            if (binding) {
                state["binding"] = binding.serialize();
            }
            if (this.$cli.quiet) {
                this.$cli.logYamlObject(state, true);
            } else {
                this.$cli.logNamedYamlObject(state, module.id, true);
            }
        }
    }

}
