import Cli from "../../../cli/Cli";
import * as _ from "lodash";
import {inject, injectable} from "inversify";
import TaskExecutor from "../../TaskExecutor";
import {execFileSync} from "child_process";
import * as Models from "../../../../domain/Models";
import TaskExecutorContext from "../../TaskExecutorContext";
import OpsApplicationConfiguration from "../../../../application/OpsApplicationConfiguration";

@injectable()
export default class TerraformSourcesFormatterTaskExecutor
    implements TaskExecutor {

    public readonly name: string = "terraform-sources-formatter";

    constructor(
        @inject(Cli) protected $cli: Cli,
        @inject(OpsApplicationConfiguration)
        readonly $configuration: OpsApplicationConfiguration,
    ) {

    }

    async execute(
        task: Models.ModuleTask,
        context: TaskExecutorContext,
        module: Models.Module,
        binding: Models.Module
    ) {
        execFileSync(
            this.$configuration.context.cmdTerraform,
            ["fmt"],
            {
                cwd: module.paths.directory,
                env: _.merge({}, process.env, module.env.toHash())
            }
        ).toString();
        this.$cli.logInfo(`Terraform have been formatted!`);
    }

}
