import * as Primitives from "../../../../domain/Primitives";
import * as _ from "lodash";
import Exception from "../../../error/Exception";
import ModulesSelector from "../../../../domain/ModulesSelector";

const Validator = require("fastest-validator");
export default class TerraformSourcesImporterTaskConfiguration {

    constructor(
        public readonly outputFilePrefix: string,
        public readonly selector: ModulesSelector
    ) {

    }

    static parse(configuration: Primitives.Serialized) {
        if (!configuration) throw new Error("Missing configuration!");
        configuration = _.merge({
            output: {
                file: {
                    prefix: "~imports"
                }
            }
        }, configuration);
        const validator: any = new Validator();
        const errors = validator.validate(configuration, {
            "output": {type: "object"},
            "selector": {type: "object"}
        } as any);
        if (errors !== true) throw new Exception("Invalid task configuration:", errors);
        return new TerraformSourcesImporterTaskConfiguration(
            configuration.output.file.prefix,
            configuration.selector
        );
    }
}
