import * as Primitives from "../../../../domain/Primitives";
import Exception from "../../../error/Exception";
import * as Models from "../../../../domain/Models";
import * as _ from "lodash";

const Validator = require("fastest-validator");

export default class TerraformTaskMoveStateFileConfiguration {
    constructor(
        readonly stateFileTarget: string,
        readonly stateFileSource: string,
        readonly stateFileMethod: "copy" | "move",
        readonly stateBucketName: string,
        readonly stateRegion: string,
        readonly stateCredentialsAccessKey: string,
        readonly stateCredentialsSecretKey: string
    ) {

    }

    static parse(
        configuration: Primitives.Serialized,
        module: Models.Module,
        binding: Models.Module
    ): TerraformTaskMoveStateFileConfiguration {
        if (!configuration) throw new Error("Missing configuration!");
        configuration = _.merge({
            state: {
                file: {
                    method: "copy",
                    target: module.context.environment.terraform.state.file
                },
                bucket: {
                    name: module.context.environment.terraform.backend.s3.bucket.name,
                    region: module.context.environment.terraform.backend.s3.region,
                    credentials: {
                        "access-key": module.context.environment.terraform.backend.s3.credentials["access-key"],
                        "secret-key": module.context.environment.terraform.backend.s3.credentials["secret-key"]
                    }
                }
            }
        }, configuration);
        const validator: any = new Validator();
        const errors = validator.validate(configuration, {
            state: {
                type: "object",
                props: {
                    file: {
                        type: "object",
                        props: {
                            target: {type: "string"},
                            source: {type: "string"},
                            method: {type: "string"},
                        }
                    },
                    bucket: {
                        type: "object",
                        props: {
                            name: {type: "string"},
                            region: {type: "string"},
                            credentials: {
                                type: "object",
                                props: {
                                    "access-key": {type: "string"},
                                    "secret-key": {type: "string"}
                                }
                            }
                        }
                    }
                }
            }
        } as any);
        if (errors !== true) throw new Exception("Invalid task configuration:", errors);
        return new TerraformTaskMoveStateFileConfiguration(
            configuration.state.file.target,
            configuration.state.file.source,
            configuration.state.file.method,
            configuration.state.bucket.name,
            configuration.state.bucket.region,
            configuration.state.bucket.credentials["access-key"],
            configuration.state.bucket.credentials["secret-key"]
        );
    }
}
