import * as Models from "../../../../domain/Models";
import * as Primitives from "../../../../domain/Primitives";
import * as _ from "lodash";
import Exception from "../../../error/Exception";

const Validator = require("fastest-validator");

export default class TerragruntConfigurationGeneratorTaskConfiguration {

    constructor(
        public readonly outputFileName: string,
        public readonly templateSection: string,
        public readonly context: Primitives.Serialized
    ) {

    }

    static parse(configuration: Primitives.Serialized, module: Models.Module, binding: Models.Module) {
        configuration = _.merge({
            output: {
                file: {
                    name: "~terraform.tf"
                }
            },
            template: {
                section: "terragrunt-configuration"
            },
            context: {}
        }, configuration);
        const validator: any = new Validator();
        const errors = validator.validate(configuration, {
            "output": {type: "object"},
            "template": {type: "object"},
            "context": {type: "object"},
        } as any);
        if (errors !== true) throw new Exception("Invalid task configuration:", errors);
        return new TerragruntConfigurationGeneratorTaskConfiguration(
            configuration.output.file.name,
            configuration.template.section,
            configuration.context
        );
    }
}
