import * as Primitives from "./Primitives";
import ModulesSelector from "./ModulesSelector";

export interface Manifest {
    module: Module,
    binding?: Module
}

export type ModuleEnv = { [key: string]: string }
export type ModuleContext = { [key: string]: any }
export type ModuleTemplates = { [key: string]: string }
export type ModuleLifecycles = { [key: string]: ModuleLifecycle } | ModuleLifecycle[]
export type ModuleTasks = { [key: string]: ModuleTask } | ModuleTask[]

export type ModuleInputParameters = {
    [key: string]: ModuleInputParameter
}

export type ModuleHelpers = {
    [key: string]: Function
}

export type ModuleInputParameterPrompt = {
    message: string,
    type: string,
    choices?: any[] | Function,
    defaults?: string | Function
}

export type ModuleInputParameter = {
    name: string,
    required: boolean,
    provided: boolean,
    value?: any,
    prompt?: ModuleInputParameterPrompt
}

export interface Module
    extends Primitives.Serialized {
    id: string,
    metadata?: ModuleMetadata,
    inputs?: ModuleInputParameters,
    helpers?: ModuleHelpers,
    paths?: ModulePaths
    env?: ModuleEnv
    lifecycles?: ModuleLifecycles
    tasks?: ModuleTasks
    context?: ModuleContext
    templates?: ModuleTemplates
}

export interface ModuleMetadata
    extends Primitives.Serialized {
    name?: string
    enabled?: boolean
    virtual?: boolean
    binding?: boolean
    schema?: string,
    $interpolation?: "none" | "in-progress" | "completed"
    $parents?: string[]
    inherit?: boolean | ModuleMetadataInherit
    tags?: string[]
}

export interface ModuleMetadataInherit {
    into?: ModulesSelector
}

export interface ModuleLifecycleAction
    extends Primitives.Serialized {
    name?: string,
    task?: ModuleLifecycleActionTask | string,
    description?: string
}

export interface ModuleLifecycleActionTask
    extends Primitives.Serialized {
    name?: string,
    configuration?: Object
}

export interface ModuleLifecycle
    extends Primitives.Serialized {
    name?: string,
    dependencies?: string[]
    actions?: Array<ModuleLifecycleAction | string>
    "actions-before"?: Array<ModuleLifecycleAction | string>
    "actions-after"?: Array<ModuleLifecycleAction | string>
}

export interface ModulePaths
    extends Primitives.Serialized {
    // Absolute path to module manifest (module.yaml)
    manifest?: string
    // Relative path to module
    module?: string
    // Absolute path module
    directory?: string
    // Basename
    unit?: string
}

export interface ModuleTask
    extends Primitives.Serialized {
    name?: string
    configuration?: Object
    metadata?: ModuleTaskMetadata
}

export interface ModuleTaskMetadata {
    enabled?: boolean
    executor?: string
    "supports-virtual"?: boolean
    description?: string
}
