export interface ArrayCollection<SELF, T>
    extends Model<SELF>, ArrayIterable<T> {

    readonly size: number

    has(identifier: string): boolean

    getAll(): T[]

}

export interface ArrayIterable<T> {
    forEach(callback: (value: T, index: number) => void): void
}

export interface Cloneable<T> {
    clone(): T
}

export interface Inheritable<T> {
    inheritFrom(source: T): void
}

export interface MapCollection<SELF, T>
    extends Model<SELF>, MapIterable<T>, MapFilterable<T, MapCollection<SELF, T>> {

    readonly size: number

    has(key: string): boolean

    toHash(): { [key: string]: T }

}

export interface MapIterable<T> {
    forEach(callback: (value: T, key: string) => void): void
}

export interface MapFilterable<T, Collection> {
    filter(callback: (value: T, key: string) => boolean): Collection
}

export interface Model<T>
    extends Serializable,
        Cloneable<T>,
        Inheritable<T> {

}

export interface Serializable {
    serialize(): Serialized
}

export interface SetCollection<SELF, T>
    extends Model<SELF>, SetIterable<T> {

    readonly size: number

    has(key: T): boolean

    getAll(): T[]

}

export interface SetIterable<T> {
    forEach(callback: (value: T) => void): void
}

export interface Serialized {
    [key: string]: any

    [index: number]: any
}
