import * as _ from "lodash";
import ModuleMetadata from "./ModuleMetadata";
import ModulePaths from "./ModulePaths";
import ModuleEnv from "./ModuleEnv";
import ModuleLifecycles from "./ModuleLifecycles";
import ModuleTasks from "./ModuleTasks";
import ModuleTemplates from "./ModuleTemplates";
import * as Primitives from "../Primitives";
import * as Manifests from "../Manifests";
import ModuleInputParameters from "./ModuleInputParameters";
import ModuleHelpers from "./ModuleHelpers";

export default class Module
    implements Primitives.Model<Module> {

    constructor(
        readonly id: string,
        readonly metadata: ModuleMetadata,
        readonly inputs: ModuleInputParameters,
        readonly paths: ModulePaths,
        readonly env: ModuleEnv,
        readonly lifecycles: ModuleLifecycles,
        readonly tasks: ModuleTasks,
        readonly templates: ModuleTemplates,
        readonly helpers: ModuleHelpers,
        public context: Primitives.Serialized,
    ) {
    }

    clone(): Module {
        return new Module(
            this.id,
            this.metadata.clone(),
            this.inputs.clone(),
            this.paths.clone(),
            this.env.clone(),
            this.lifecycles.clone(),
            this.tasks.clone(),
            this.templates.clone(),
            this.helpers.clone(),
            _.cloneDeep(this.context),
        );
    }

    serialize(): Manifests.Module {
        const serialized: Manifests.Module = {
            id: this.id,
            metadata: this.metadata.serialize(),
            inputs: this.inputs.serialize(),
            paths: this.paths.serialize()
        };
        if (this.env.size) serialized.env = this.env.serialize();
        if (Object.keys(this.context).length) serialized.context = _.cloneDeep(this.context);
        if (this.lifecycles.size) serialized.lifecycles = this.lifecycles.serialize();
        if (this.tasks.size) serialized.tasks = this.tasks.serialize();
        if (this.templates.size) serialized.templates = this.templates.serialize();
        if (this.inputs.size) serialized.inputs = this.inputs.serialize();
        if (this.helpers.size) serialized.helpers = this.helpers.serialize();
        return serialized;
    }

    serializeAsBinding(): Manifests.Module {
        const serialized: Manifests.Module = {
            id: this.id,
            metadata: this.metadata.serialize(),
            inputs: this.inputs.serialize(),
            paths: this.paths.serialize()
        };
        if (this.env.size) serialized.env = this.env.serialize();
        if (Object.keys(this.context).length) serialized.context = _.cloneDeep(this.context);
        if (this.templates.size) serialized.templates = this.templates.serialize();
        if (this.inputs.size) serialized.inputs = this.inputs.serialize();
        if (this.helpers.size) serialized.helpers = this.helpers.serialize();
        return serialized;
    }

    inheritFrom(source: Module): void {
        if (this.metadata.parents.has(source.id)) return;
        this.env.inheritFrom(source.env);
        this.inputs.inheritFrom(source.inputs);
        this.context = _.merge({}, source.context, this.context);
        this.lifecycles.inheritFrom(source.lifecycles);
        this.tasks.inheritFrom(source.tasks);
        this.templates.inheritFrom(source.templates);
        this.helpers.inheritFrom(source.helpers);
        this.metadata.inheritFrom(source.metadata);
        this.metadata.parents.add(source.id);
    }
}
