import * as Primitives from "../Primitives";
import YamlLazyValue from "../../components/yaml/YamlLazyValue";

export default class ModuleEnv
    implements Primitives.MapCollection<ModuleEnv, string | YamlLazyValue> {

    constructor(
        private readonly variables: Map<string, string | YamlLazyValue> = new Map<string, string | YamlLazyValue>()
    ) {
    }

    get size(): number {
        return this.variables.size;
    }

    clone(): ModuleEnv {
        return new ModuleEnv(
            new Map(Array.from(this.variables.entries()))
        );
    }

    forEach(callback: (value: string | YamlLazyValue, key: string) => void): void {
        this.variables.forEach(callback);
    }

    get(name: string): string {
        if (!this.has(name)) throw new Error(`Environment variable ${name} does not exists!`);
        return this.variables.get(name) as string;
    }

    has(key: string): boolean {
        return this.variables.has(key);
    }

    inheritFrom(source: ModuleEnv): void {
        source.forEach((value, name) => !this.has(name) && this.variables.set(name, value));
    }

    serialize(): Primitives.Serialized {
        const serialized: Primitives.Serialized = {};
        this.forEach((value: string| YamlLazyValue, name) => {
            if (value instanceof YamlLazyValue) {
                serialized[name] = value.toString();
            } else {
                serialized[name] = value;
            }
        });
        return serialized;
    }

    toHash(): Primitives.Serialized {
        return this.serialize()
    }

    filter(filter: (value: (string | YamlLazyValue), key: string) => boolean): ModuleEnv {
        const map = new Map<string, string | YamlLazyValue>();
        this.forEach((value: (string | YamlLazyValue), key: string) => {
            if (filter(value, key)) {
                map.set(key, value);
            }
        });
        return new ModuleEnv(map);
    }

}
