import * as Primitives from "../Primitives";
import YamlLazyValue from "../../components/yaml/YamlLazyValue";

export default class ModuleHelpers
    implements Primitives.MapCollection<ModuleHelpers, Function> {

    constructor(
        private readonly helpers: Map<string, Function> = new Map<string, Function>()
    ) {
    }

    get size(): number {
        return this.helpers.size;
    }

    get entries(): Array<[string, Function]> {
        return Array.from(this.helpers.entries());
    }

    clone(): ModuleHelpers {
        return new ModuleHelpers(
            new Map(Array.from(this.helpers.entries()))
        );
    }

    forEach(callback: (value: Function, key: string) => void): void {
        this.helpers.forEach(callback);
    }

    get(name: string): Function {
        if (!this.has(name)) throw new Error(`Helper ${name} does not exists!`);
        return this.helpers.get(name) as Function;
    }

    has(key: string): boolean {
        return this.helpers.has(key);
    }

    inheritFrom(source: ModuleHelpers): void {
        source.forEach((value, name) => !this.has(name) && this.helpers.set(name, value));
    }

    serialize(): Primitives.Serialized {
        const serialized: Primitives.Serialized = {};
        this.forEach((value, name) => serialized[name] = value);
        return serialized;
    }

    toHash(): Primitives.Serialized {
        return this.serialize()
    }

    filter(filter: (value: Function, key: string) => boolean): ModuleHelpers {
        const map = new Map<string, Function>();
        this.forEach((value: Function, key: string) => {
            if (filter(value, key)) {
                map.set(key, value);
            }
        });
        return new ModuleHelpers(map);
    }

}
