import * as Primitives from "../Primitives";
import ModuleInputParameter from "./ModuleInputParameter";

export default class ModuleInputParameters
    implements Primitives.MapCollection<ModuleInputParameters, ModuleInputParameter> {

    constructor(
        private readonly inputs: Map<string, ModuleInputParameter> = new Map<string, ModuleInputParameter>()
    ) {
    }

    get size(): number {
        return this.inputs.size;
    }

    get entries(): Array<[string, ModuleInputParameter]> {
        return Array.from(this.inputs.entries());
    }

    clone(): ModuleInputParameters {
        return new ModuleInputParameters(
            new Map(Array.from(this.inputs.entries()))
        );
    }

    forEach(callback: (value: ModuleInputParameter, key: string) => void): void {
        this.inputs.forEach(callback);
    }

    get(name: string): ModuleInputParameter {
        if (!this.has(name)) throw new Error(`Input variable ${name} does not exists!`);
        return this.inputs.get(name) as ModuleInputParameter;
    }

    has(key: string): boolean {
        return this.inputs.has(key);
    }

    inheritFrom(source: ModuleInputParameters): void {
        source.forEach((value, name) => !this.has(name) && this.inputs.set(name, value));
    }

    serialize(): Primitives.Serialized {
        const serialized: Primitives.Serialized = {};
        this.forEach((value, name) => serialized[name] = value.serialize());
        return serialized;
    }

    toHash(): Primitives.Serialized {
        return this.serialize()
    }

    filter(filter: (value: ModuleInputParameter, key: string) => boolean): ModuleInputParameters {
        const map = new Map<string, ModuleInputParameter>();
        this.forEach((value: ModuleInputParameter, key: string) => {
            if (filter(value, key)) {
                map.set(key, value);
            }
        });
        return new ModuleInputParameters(map);
    }

}
