import ModuleLifecycleActions from "./ModuleLifecycleActions";
import * as Primitives from "../Primitives";

export default class ModuleLifecycle
    implements Primitives.Model<ModuleLifecycle> {

    constructor(
        readonly name: string,
        readonly dependencies: string[],
        readonly actions: ModuleLifecycleActions,
        readonly actionsBefore: ModuleLifecycleActions,
        readonly actionsAfter: ModuleLifecycleActions
    ) {

    }

    inheritFrom(lifecycle: ModuleLifecycle): void {
        this.actions.inheritFrom(lifecycle.actions, {prepend: false});
        this.actionsBefore.inheritFrom(lifecycle.actionsBefore, {prepend: false});
        this.actionsAfter.inheritFrom(lifecycle.actionsAfter);
    }

    clone(): ModuleLifecycle {
        return new ModuleLifecycle(
            this.name,
            Array.from(this.dependencies),
            this.actions.clone(),
            this.actionsBefore.clone(),
            this.actionsAfter.clone()
        )
    }

    serialize(): Primitives.Serialized {
        const serialized: Primitives.Serialized = {name: this.name};
        if (this.dependencies.length) serialized.dependencies = this.dependencies;
        if (this.actions.size) serialized.actions = this.actions.serialize();
        if (this.actionsBefore.size) serialized["actions-before"] = this.actionsBefore.serialize();
        if (this.actionsAfter.size) serialized["actions-after"] = this.actionsAfter.serialize();
        return serialized;
    }
}
