import ModuleLifecycleAction from "./ModuleLifecycleAction";
import * as Primitives from "../Primitives";

export default class ModuleLifecycleActions
    implements Primitives.ArrayCollection<ModuleLifecycleActions, ModuleLifecycleAction> {

    constructor(
        private readonly actions: ModuleLifecycleAction[] = new Array<ModuleLifecycleAction>()
    ) {

    }

    get size(): number {
        return this.actions.length;
    }

    getNames(): string[] {
        return this.actions.map((action) => action.name);
    }

    get(name: string): ModuleLifecycleAction {
        if (!this.has(name)) throw new Error(`Action '${name}' does not exists!`);
        return this.lookup(name) as ModuleLifecycleAction
    }

    has(name: string): boolean {
        return this.lookup(name) != undefined
    }

    clone(): ModuleLifecycleActions {
        const actions = new Array<ModuleLifecycleAction>();
        this.forEach((action) => actions.push(action));
        return new ModuleLifecycleActions(actions);
    }

    forEach(callback: (action: ModuleLifecycleAction, index: number) => void): void {
        this.actions.forEach(callback);
    }

    inheritFrom(actions: ModuleLifecycleActions,
                configuration: { prepend: boolean } = {prepend: false}): void {
        const newActions: ModuleLifecycleAction[] = [];
        actions.forEach((action) => {
            if (this.has(action.name)) {
                const target = this.get(action.name);
                target.inheritFrom(action);
            } else {
                newActions.push(action.clone());
            }
        });
        if (configuration.prepend) {
            this.prepend(...newActions);
        } else {
            this.append(...newActions);
        }
    }

    serialize(): Primitives.Serialized {
        const serialized: Primitives.Serialized = [];
        this.forEach((action, index) => {
            const serializedAction = action.serialize();
            if (serializedAction.name && Object.keys(serializedAction).length == 1) {
                serialized[index] = serializedAction.name;
            } else {
                serialized[index] = serializedAction;
            }
        });
        return serialized;
    }

    getAll(): ModuleLifecycleAction[] {
        return Array.from(this.actions.values());
    }

    protected lookup(name: string): ModuleLifecycleAction | undefined {
        return this.actions.find((action) => action.name == name)
    }

    protected prepend(...actions: ModuleLifecycleAction[]) {
        this.actions.unshift(...actions)
    }

    protected append(...action: ModuleLifecycleAction[]) {
        this.actions.push(...action)
    }
}
