import ModuleLifecycle from "./ModuleLifecycle";
import * as Primitives from "../Primitives";

export default class ModuleLifecycles
    implements Primitives.MapCollection<ModuleLifecycles, ModuleLifecycle> {
    constructor(
        private readonly lifecycles: Map<string, ModuleLifecycle> = new Map<string, ModuleLifecycle>()
    ) {
    }

    get size(): number {
        return this.lifecycles.size
    }

    get(name: string): ModuleLifecycle {
        if (!this.has(name)) throw new Error(`Lifecycle '${name}' does not exists!`);
        return this.lifecycles.get(name) as ModuleLifecycle
    }

    has(name: string): boolean {
        return this.lifecycles.has(name);
    }

    getNames(): string[] {
        return Array.from(this.lifecycles.keys());
    }

    forEach(callback: (lifecycle: ModuleLifecycle, name: string) => void): void {
        this.lifecycles.forEach(callback);
    }

    inheritFrom(lifecycles: ModuleLifecycles): void {
        lifecycles.forEach((lifecycle) => {
            if (this.has(lifecycle.name)) {
                const target = this.get(lifecycle.name);
                target.inheritFrom(lifecycle);
            } else {
                this.lifecycles.set(lifecycle.name, lifecycle.clone())
            }
        });
    }

    clone(): ModuleLifecycles {
        const lifecycles = new Map<string, ModuleLifecycle>();
        this.lifecycles.forEach((lifecycle) => lifecycles.set(lifecycle.name, lifecycle));
        return new ModuleLifecycles(lifecycles);
    }

    serialize(): Primitives.Serialized {
        const serialized: Primitives.Serialized = {};
        this.forEach((lifecycle) => {
            serialized[lifecycle.name] = lifecycle.serialize();
            delete serialized[lifecycle.name].name
        });
        return serialized;
    }

    toHash(): { [name: string]: ModuleLifecycle } {
        const hash: { [name: string]: ModuleLifecycle } = {};
        if (this.lifecycles.size)
            this.lifecycles.forEach((lifecycle, name) => hash[name] = lifecycle);
        return hash;

    }

    filter(filter: (value: ModuleLifecycle, key: string) => boolean): ModuleLifecycles {
        const map = new Map<string, ModuleLifecycle>();
        this.forEach((value: ModuleLifecycle, key: string) => {
            if (filter(value, key)) {
                map.set(key, value);
            }
        });
        return new ModuleLifecycles(map);
    }
}
