import ModuleMetadataInheritance from "./ModuleMetadataInheritance";
import * as Primitives from "../Primitives";

export default class ModuleMetadata
    implements Primitives.Model<ModuleMetadata> {

    constructor(
        readonly enabled: boolean,
        readonly virtual: boolean,
        readonly binding: boolean,
        readonly inherit: ModuleMetadataInheritance,
        readonly tags: Set<string>,
        readonly schema: string,
        readonly autoload: Set<string>,
        public interpolation: "none" | "in-progress" | "completed",
        readonly parents: Set<string>,
    ) {

    }

    clone(): ModuleMetadata {
        return new ModuleMetadata(
            this.enabled,
            this.virtual,
            this.binding,
            this.inherit.clone(),
            new Set<string>(Array.from(this.tags)),
            this.schema,
            this.autoload,
            this.interpolation,
            new Set<string>(Array.from(this.parents))
        );
    }

    inheritFrom(source: ModuleMetadata): void {
    }

    serialize(): Primitives.Serialized {
        const serialized: Primitives.Serialized = {};
        const inherit = this.inherit.serialize();
        if (!this.enabled) serialized.enabled = false;
        if (this.virtual) serialized.virtual = true;
        if (this.binding) serialized.binding = true;
        if (this.tags.size) serialized.tags = Array.from(this.tags);
        serialized.schema = this.schema;
        if (Object.keys(inherit).length) serialized.inherit = inherit;
        if (this.autoload) serialized.autoload = Array.from(this.autoload);
        return serialized;
    }
}
