import * as Primitives from "../Primitives";

export default class ModuleMetadataTags
    implements Primitives.SetCollection<ModuleMetadataTags, string> {

    constructor(
        private readonly content = new Set<string>()
    ) {
    }

    get size() {
        return this.content.size;
    }

    getAll() {
        return Array.from(this.content);
    }

    has(name: string): boolean {
        return this.content.has(name);
    }

    forEach(callback: (tag: string) => void): void {
        this.content.forEach(callback);
    }

    inheritFrom(source: ModuleMetadataTags): void {
        source.forEach((tag) => this.content.add(tag));
    }

    clone(): ModuleMetadataTags {
        const content = new Set<string>();
        this.forEach((tag) => content.add(tag));
        return new ModuleMetadataTags(content);
    }

    serialize(): Primitives.Serialized {
        return Array.from(this.content);
    }

}
