import * as _ from "lodash";
import * as Primitives from "../Primitives";
import * as Manifests from "../Manifests";
import ModuleTaskMetadata from "./ModuleTaskMetadata";

export default class ModuleTask
    implements Primitives.Model<ModuleTask> {

    constructor(
        public readonly name: string,
        public readonly metadata: ModuleTaskMetadata,
        public configuration: Primitives.Serialized = {}
    ) {
    }

    clone(): ModuleTask {
        return new ModuleTask(
            this.name,
            this.metadata.clone(),
            _.cloneDeep(this.configuration)
        );
    }

    serialize(): Manifests.ModuleTask {
        const serialized: Manifests.ModuleTask = {
            name: this.name,
            metadata: this.metadata.serialize()
        };
        if (this.configuration && Object.keys(this.configuration).length)
            serialized.configuration = this.configuration;
        return serialized;
    }

    inheritFrom(source: ModuleTask): void {
        this.configuration = _.merge({}, source.configuration, this.configuration);
        this.metadata.inheritFrom(source.metadata);
    }
}
