import ModuleTask from "./ModuleTask";
import * as Primitives from "../Primitives";

export default class ModuleTasks
    implements Primitives.MapCollection<ModuleTasks, ModuleTask> {

    constructor(
        private readonly tasks: Map<string, ModuleTask> = new Map<string, ModuleTask>()
    ) {

    }

    get size(): number {
        return this.tasks.size;
    }

    get(name: string): ModuleTask {
        if (!this.has(name)) throw new Error(`Task '${name}' does not exists!`);
        return this.tasks.get(name) as ModuleTask
    }

    has(name: string): boolean {
        return this.tasks.has(name);
    }

    forEach(callback: (task: ModuleTask, name: string) => void): void {
        this.tasks.forEach(callback);
    }

    inheritFrom(tasks: ModuleTasks): void {
        tasks.forEach((task) => {
            if (this.has(task.name)) {
                this.get(task.name).inheritFrom(task);
            } else {
                this.addTask(task.clone());
            }
        });
    }

    clone(): ModuleTasks {
        const content = new Map<string, ModuleTask>();
        this.forEach((task) => content.set(task.name, task));
        return new ModuleTasks(content);
    }

    serialize(): Primitives.Serialized {
        const serialized: Primitives.Serialized = {};

        if (this.tasks.size)
            this.tasks.forEach((task, name) => serialized[name] = task.serialize());

        return serialized;
    }

    toHash(): { [name: string]: ModuleTask } {

        const hash: { [name: string]: ModuleTask } = {};

        if (this.tasks.size)
            this.tasks.forEach((task, name) => hash[name] = task);

        return hash;

    }

    private addTask(task: ModuleTask) {
        this.tasks.set(task.name, task.clone());
    }

    filter(filter: (value: ModuleTask, key: string) => boolean): ModuleTasks {
        const map = new Map<string, ModuleTask>();
        this.forEach((value: ModuleTask, key: string) => {
            if (filter(value, key)) {
                map.set(key, value);
            }
        });
        return new ModuleTasks(map);
    }

}
