import * as Primitives from "../Primitives";
import * as _ from "lodash";

export default class ModuleTemplates
    implements Primitives.MapCollection<ModuleTemplates, any> {

    constructor(
        private readonly templates: Map<string, any> = new Map<string, any>()
    ) {

    }

    get size(): number {
        return this.templates.size
    }

    get(name: string): any {
        if (!this.has(name)) throw new Error(`Template '${name}' does not exists!`);
        return this.templates.get(name) as any
    }

    has(name: string): boolean {
        return this.templates.has(name);
    }

    getNames(): string[] {
        return Array.from(this.templates.keys());
    }

    getAll() {
        return Array.from(this.templates.entries()).map((entry) => {
            return {
                name: entry[0],
                content: entry[1]
            }
        });
    }

    forEach(callback: (content: any, name: string) => void): void {
        this.templates.forEach(callback);
    }

    inheritFrom(templates: ModuleTemplates): void {
        templates.forEach((content, name) => {
            if (this.has(name)) {
                const template = this.templates.get(name);
                if ((template instanceof Object) && (content instanceof Object)) {
                    this.templates.set(name, _.merge({}, template, content));
                }
            } else {
                this.templates.set(name, content)
            }
        });
    }

    clone(): ModuleTemplates {
        const templates = new Map<string, any>();
        this.forEach((content, name) => templates.set(name, content));
        return new ModuleTemplates(templates);
    }

    serialize(): Primitives.Serialized {
        const serialized: Primitives.Serialized = {};
        this.forEach((content, name) => serialized[name] = content);
        return serialized;
    }

    toHash(): Primitives.Serialized {
        return this.serialize();
    }

    filter(filter: (value: any, key: string) => boolean): ModuleTemplates {
        const map = new Map<string, any>();
        this.forEach((value: any, key: string) => {
            if (filter(value, key)) {
                map.set(key, value);
            }
        });
        return new ModuleTemplates(map);
    }

}
