import {Container} from "inversify";
import OpsApplication from "./application/OpsApplication";
import InterpolateApplication from "./application/InterpolateApplication";
import OpsApplicationContextReader from "./application/OpsApplicationContextReader";
import InterpolateApplicationContextReader from "./application/InterpolateApplicationContextReader";
import Cli from "./components/cli/Cli";
import OpsApplicationConfiguration from "./application/OpsApplicationConfiguration";
import InterpolateApplicationConfiguration from "./application/InterpolateApplicationConfiguration";
import OpsApplicationUI from "./application/OpsApplicationUI";
import InterpolateApplicationUI from "./application/InterpolateApplicationUI";
import ManifestsScanner from "./components/manifests/ManifestsScanner";
import EchoTaskExecutor from "./components/tasks/executors/echo/EchoTaskExecutor";
import FSTaskExecutor from "./components/tasks/executors/fs/FSTaskExecutor";
import ManifestTaskExecutor from "./components/tasks/executors/manifest/ManifestTaskExecutor";
import TaskExecutionService from "./components/tasks/TaskExecutionService";
import TerraformSourcesFormatterTaskExecutor
    from "./components/tasks/executors/terraform/TerraformSourcesFormatterTaskExecutor";
import TerragruntConfigurationGeneratorTaskExecutor
    from "./components/tasks/executors/terraform/TerragruntConfigurationGeneratorTaskExecutor";
import TerraformTaskExecutor from "./components/tasks/executors/terraform/TerraformTaskExecutor";
import TerraformSourcesImporterTaskExecutor
    from "./components/tasks/executors/terraform/TerraformSourcesImporterTaskExecutor";
import OpsTaskExecutor from "./components/tasks/executors/ops/OpsTaskExecutor";
import TerraformVariablesGeneratorTaskExecutor
    from "./components/tasks/executors/terraform/TerraformVariablesGeneratorTaskExecutor";
import SchemaValidator from "./domain/SchemaValidator";
import ModelsBuilder from "./domain/ModelsBuilder";
import ModulesService from "./components/modules/ModulesService";
import TerraformSectionSourcesGeneratorTaskExecutor
    from "./components/tasks/executors/terraform/TerraformSectionSourcesGeneratorTaskExecutor";
import TerraformTemplateSourcesGeneratorTaskExecutor
    from "./components/tasks/executors/terraform/TerraformTemplateSourcesGeneratorTaskExecutor";
import TerraformRemoteStatesGeneratorTaskExecutor
    from "./components/tasks/executors/terraform/TerraformRemoteStatesGeneratorTaskExecutor";
import ExecTaskExecutor from "./components/tasks/executors/exec/ExecTaskExecutor";
import YamlService from "./components/yaml/YamlService";

const container = new Container();
container.bind<Cli>(Cli).toSelf().inSingletonScope();
container.bind<OpsApplication>(OpsApplication).toSelf().inSingletonScope();
container.bind<InterpolateApplication>(InterpolateApplication).toSelf().inSingletonScope();
container.bind<OpsApplicationUI>(OpsApplicationUI).toSelf().inSingletonScope();
container.bind<InterpolateApplicationUI>(InterpolateApplicationUI).toSelf().inSingletonScope();
container.bind<OpsApplicationContextReader>(OpsApplicationContextReader).toSelf().inSingletonScope();
container.bind<InterpolateApplicationContextReader>(InterpolateApplicationContextReader).toSelf().inSingletonScope();
container.bind<OpsApplicationConfiguration>(OpsApplicationConfiguration).toSelf().inSingletonScope();
container.bind<InterpolateApplicationConfiguration>(InterpolateApplicationConfiguration).toSelf().inSingletonScope();
container.bind<ModulesService>(ModulesService).toSelf().inSingletonScope();
container.bind<ManifestsScanner>(ManifestsScanner).toSelf().inSingletonScope();
container.bind<SchemaValidator>(SchemaValidator).toSelf().inSingletonScope();
container.bind<ModelsBuilder>(ModelsBuilder).toSelf().inSingletonScope();
container.bind<YamlService>(YamlService).toSelf().inSingletonScope();
container.bind<TaskExecutionService>(TaskExecutionService).toSelf().inSingletonScope();
container.bind<OpsTaskExecutor>(OpsTaskExecutor).toSelf().inSingletonScope();
container.bind<FSTaskExecutor>(FSTaskExecutor).toSelf().inSingletonScope();
container.bind<ManifestTaskExecutor>(ManifestTaskExecutor).toSelf().inSingletonScope();
container.bind<EchoTaskExecutor>(EchoTaskExecutor).toSelf().inSingletonScope();
container.bind<ExecTaskExecutor>(ExecTaskExecutor).toSelf().inSingletonScope();
container.bind<TerraformVariablesGeneratorTaskExecutor>(TerraformVariablesGeneratorTaskExecutor).toSelf().inSingletonScope();
container.bind<TerraformRemoteStatesGeneratorTaskExecutor>(TerraformRemoteStatesGeneratorTaskExecutor).toSelf().inSingletonScope();
container.bind<TerraformSectionSourcesGeneratorTaskExecutor>(TerraformSectionSourcesGeneratorTaskExecutor).toSelf().inSingletonScope();
container.bind<TerraformTemplateSourcesGeneratorTaskExecutor>(TerraformTemplateSourcesGeneratorTaskExecutor).toSelf().inSingletonScope();
container.bind<TerraformSourcesImporterTaskExecutor>(TerraformSourcesImporterTaskExecutor).toSelf().inSingletonScope();
container.bind<TerraformSourcesFormatterTaskExecutor>(TerraformSourcesFormatterTaskExecutor).toSelf().inSingletonScope();
container.bind<TerragruntConfigurationGeneratorTaskExecutor>(TerragruntConfigurationGeneratorTaskExecutor).toSelf().inSingletonScope();
container.bind<TerraformTaskExecutor>(TerraformTaskExecutor).toSelf().inSingletonScope();
export default container;
